"""Validation helpers for router/ospf6 - Auto-generated"""

from typing import Any, TypedDict, Literal
from typing_extensions import NotRequired

# Import common validators from central _helpers module
from hfortix_fortios._helpers import (
    validate_enable_disable,
    validate_integer_range,
    validate_string_length,
    validate_port_number,
    validate_ip_address,
    validate_ipv6_address,
    validate_mac_address,
)

# Import central validation functions (avoid duplication across 1,062 files)
from hfortix_fortios._helpers.validation import (
    validate_required_fields as _validate_required_fields,
    validate_enum_field as _validate_enum_field,
    validate_query_parameter as _validate_query_parameter,
)

# ============================================================================
# Required Fields Validation
# Auto-generated from schema
# ============================================================================

# ⚠️  IMPORTANT: FortiOS schemas have known issues with required field marking:

# Do NOT use this list for strict validation - test with the actual FortiOS API!

# Fields marked as required (after filtering false positives)
REQUIRED_FIELDS = [
]

# Fields with defaults (optional)
FIELDS_WITH_DEFAULTS = {
    "abr-type": "standard",
    "auto-cost-ref-bandwidth": 1000,
    "default-information-originate": "disable",
    "log-neighbour-changes": "enable",
    "default-information-metric": 10,
    "default-information-metric-type": "2",
    "default-information-route-map": "",
    "default-metric": 10,
    "router-id": "0.0.0.0",
    "spf-timers": "",
    "bfd": "disable",
    "restart-mode": "none",
    "restart-period": 120,
    "restart-on-topology-change": "disable",
}

# ============================================================================
# Deprecated Fields
# Auto-generated from schema - warns users about deprecated fields
# ============================================================================

# Deprecated fields with migration guidance
DEPRECATED_FIELDS = {
}

# ============================================================================
# Field Metadata (Type Information & Descriptions)
# Auto-generated from schema - use for IDE autocomplete and documentation
# ============================================================================

# Field types mapping
FIELD_TYPES = {
    "abr-type": "option",  # Area border router type.
    "auto-cost-ref-bandwidth": "integer",  # Reference bandwidth in terms of megabits per second.
    "default-information-originate": "option",  # Enable/disable generation of default route.
    "log-neighbour-changes": "option",  # Log OSPFv3 neighbor changes.
    "default-information-metric": "integer",  # Default information metric.
    "default-information-metric-type": "option",  # Default information metric type.
    "default-information-route-map": "string",  # Default information route map.
    "default-metric": "integer",  # Default metric of redistribute routes.
    "router-id": "ipv4-address-any",  # A.B.C.D, in IPv4 address format.
    "spf-timers": "user",  # SPF calculation frequency.
    "bfd": "option",  # Enable/disable Bidirectional Forwarding Detection (BFD).
    "restart-mode": "option",  # OSPFv3 restart mode (graceful or none).
    "restart-period": "integer",  # Graceful restart period in seconds.
    "restart-on-topology-change": "option",  # Enable/disable continuing graceful restart upon topology cha
    "area": "string",  # OSPF6 area configuration.
    "ospf6-interface": "string",  # OSPF6 interface configuration.
    "redistribute": "string",  # Redistribute configuration.
    "passive-interface": "string",  # Passive interface configuration.
    "summary-address": "string",  # IPv6 address summary configuration.
}

# Field descriptions (help text from FortiOS API)
FIELD_DESCRIPTIONS = {
    "abr-type": "Area border router type.",
    "auto-cost-ref-bandwidth": "Reference bandwidth in terms of megabits per second.",
    "default-information-originate": "Enable/disable generation of default route.",
    "log-neighbour-changes": "Log OSPFv3 neighbor changes.",
    "default-information-metric": "Default information metric.",
    "default-information-metric-type": "Default information metric type.",
    "default-information-route-map": "Default information route map.",
    "default-metric": "Default metric of redistribute routes.",
    "router-id": "A.B.C.D, in IPv4 address format.",
    "spf-timers": "SPF calculation frequency.",
    "bfd": "Enable/disable Bidirectional Forwarding Detection (BFD).",
    "restart-mode": "OSPFv3 restart mode (graceful or none).",
    "restart-period": "Graceful restart period in seconds.",
    "restart-on-topology-change": "Enable/disable continuing graceful restart upon topology change.",
    "area": "OSPF6 area configuration.",
    "ospf6-interface": "OSPF6 interface configuration.",
    "redistribute": "Redistribute configuration.",
    "passive-interface": "Passive interface configuration.",
    "summary-address": "IPv6 address summary configuration.",
}

# Field constraints (string lengths, integer ranges)
FIELD_CONSTRAINTS = {
    "auto-cost-ref-bandwidth": {"type": "integer", "min": 1, "max": 1000000},
    "default-information-metric": {"type": "integer", "min": 1, "max": 16777214},
    "default-information-route-map": {"type": "string", "max_length": 35},
    "default-metric": {"type": "integer", "min": 1, "max": 16777214},
    "restart-period": {"type": "integer", "min": 1, "max": 3600},
}

# Nested schemas (for table/list fields with children)
NESTED_SCHEMAS = {
    "area": {
        "id": {
            "type": "ipv4-address-any",
            "help": "Area entry IP address.",
            "default": "0.0.0.0",
        },
        "default-cost": {
            "type": "integer",
            "help": "Summary default cost of stub or NSSA area.",
            "default": 10,
            "min_value": 0,
            "max_value": 16777215,
        },
        "nssa-translator-role": {
            "type": "option",
            "help": "NSSA translator role type.",
            "default": "candidate",
            "options": ["candidate", "never", "always"],
        },
        "stub-type": {
            "type": "option",
            "help": "Stub summary setting.",
            "default": "summary",
            "options": ["no-summary", "summary"],
        },
        "type": {
            "type": "option",
            "help": "Area type setting.",
            "default": "regular",
            "options": ["regular", "nssa", "stub"],
        },
        "nssa-default-information-originate": {
            "type": "option",
            "help": "Enable/disable originate type 7 default into NSSA area.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "nssa-default-information-originate-metric": {
            "type": "integer",
            "help": "OSPFv3 default metric.",
            "default": 10,
            "min_value": 0,
            "max_value": 16777214,
        },
        "nssa-default-information-originate-metric-type": {
            "type": "option",
            "help": "OSPFv3 metric type for default routes.",
            "default": "2",
            "options": ["1", "2"],
        },
        "nssa-redistribution": {
            "type": "option",
            "help": "Enable/disable redistribute into NSSA area.",
            "default": "enable",
            "options": ["enable", "disable"],
        },
        "authentication": {
            "type": "option",
            "help": "Authentication mode.",
            "default": "none",
            "options": ["none", "ah", "esp"],
        },
        "key-rollover-interval": {
            "type": "integer",
            "help": "Key roll-over interval.",
            "default": 300,
            "min_value": 300,
            "max_value": 216000,
        },
        "ipsec-auth-alg": {
            "type": "option",
            "help": "Authentication algorithm.",
            "default": "md5",
            "options": ["md5", "sha1", "sha256", "sha384", "sha512"],
        },
        "ipsec-enc-alg": {
            "type": "option",
            "help": "Encryption algorithm.",
            "default": "null",
            "options": ["null", "des", "3des", "aes128", "aes192", "aes256"],
        },
        "ipsec-keys": {
            "type": "string",
            "help": "IPsec authentication and encryption keys.",
        },
        "range": {
            "type": "string",
            "help": "OSPF6 area range configuration.",
        },
        "virtual-link": {
            "type": "string",
            "help": "OSPF6 virtual link configuration.",
        },
    },
    "ospf6-interface": {
        "name": {
            "type": "string",
            "help": "Interface entry name.",
            "default": "",
            "max_length": 35,
        },
        "area-id": {
            "type": "ipv4-address-any",
            "help": "A.B.C.D, in IPv4 address format.",
            "required": True,
            "default": "0.0.0.0",
        },
        "interface": {
            "type": "string",
            "help": "Configuration interface name.",
            "required": True,
            "default": "",
            "max_length": 15,
        },
        "retransmit-interval": {
            "type": "integer",
            "help": "Retransmit interval.",
            "default": 5,
            "min_value": 1,
            "max_value": 65535,
        },
        "transmit-delay": {
            "type": "integer",
            "help": "Transmit delay.",
            "default": 1,
            "min_value": 1,
            "max_value": 65535,
        },
        "cost": {
            "type": "integer",
            "help": "Cost of the interface, value range from 0 to 65535, 0 means auto-cost.",
            "default": 0,
            "min_value": 0,
            "max_value": 65535,
        },
        "priority": {
            "type": "integer",
            "help": "Priority.",
            "default": 1,
            "min_value": 0,
            "max_value": 255,
        },
        "dead-interval": {
            "type": "integer",
            "help": "Dead interval.",
            "default": 0,
            "min_value": 1,
            "max_value": 65535,
        },
        "hello-interval": {
            "type": "integer",
            "help": "Hello interval.",
            "default": 0,
            "min_value": 1,
            "max_value": 65535,
        },
        "status": {
            "type": "option",
            "help": "Enable/disable OSPF6 routing on this interface.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "network-type": {
            "type": "option",
            "help": "Network type.",
            "default": "broadcast",
            "options": ["broadcast", "point-to-point", "non-broadcast", "point-to-multipoint", "point-to-multipoint-non-broadcast"],
        },
        "bfd": {
            "type": "option",
            "help": "Enable/disable Bidirectional Forwarding Detection (BFD).",
            "default": "global",
            "options": ["global", "enable", "disable"],
        },
        "mtu": {
            "type": "integer",
            "help": "MTU for OSPFv3 packets.",
            "default": 0,
            "min_value": 576,
            "max_value": 65535,
        },
        "mtu-ignore": {
            "type": "option",
            "help": "Enable/disable ignoring MTU field in DBD packets.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "authentication": {
            "type": "option",
            "help": "Authentication mode.",
            "default": "area",
            "options": ["none", "ah", "esp", "area"],
        },
        "key-rollover-interval": {
            "type": "integer",
            "help": "Key roll-over interval.",
            "default": 300,
            "min_value": 300,
            "max_value": 216000,
        },
        "ipsec-auth-alg": {
            "type": "option",
            "help": "Authentication algorithm.",
            "default": "md5",
            "options": ["md5", "sha1", "sha256", "sha384", "sha512"],
        },
        "ipsec-enc-alg": {
            "type": "option",
            "help": "Encryption algorithm.",
            "default": "null",
            "options": ["null", "des", "3des", "aes128", "aes192", "aes256"],
        },
        "ipsec-keys": {
            "type": "string",
            "help": "IPsec authentication and encryption keys.",
        },
        "neighbor": {
            "type": "string",
            "help": "OSPFv3 neighbors are used when OSPFv3 runs on non-broadcast media.",
        },
    },
    "redistribute": {
        "name": {
            "type": "string",
            "help": "Redistribute name.",
            "required": True,
            "default": "",
            "max_length": 35,
        },
        "status": {
            "type": "option",
            "help": "Status.",
            "default": "disable",
            "options": ["enable", "disable"],
        },
        "metric": {
            "type": "integer",
            "help": "Redistribute metric setting.",
            "default": 0,
            "min_value": 0,
            "max_value": 16777214,
        },
        "routemap": {
            "type": "string",
            "help": "Route map name.",
            "default": "",
            "max_length": 35,
        },
        "metric-type": {
            "type": "option",
            "help": "Metric type.",
            "default": "2",
            "options": ["1", "2"],
        },
    },
    "passive-interface": {
        "name": {
            "type": "string",
            "help": "Passive interface name.",
            "required": True,
            "default": "",
            "max_length": 79,
        },
    },
    "summary-address": {
        "id": {
            "type": "integer",
            "help": "Summary address entry ID.",
            "default": 0,
            "min_value": 0,
            "max_value": 4294967295,
        },
        "prefix6": {
            "type": "ipv6-network",
            "help": "IPv6 prefix.",
            "required": True,
            "default": "::/0",
        },
        "advertise": {
            "type": "option",
            "help": "Enable/disable advertise status.",
            "default": "enable",
            "options": ["disable", "enable"],
        },
        "tag": {
            "type": "integer",
            "help": "Tag value.",
            "default": 0,
            "min_value": 0,
            "max_value": 4294967295,
        },
    },
}


# Valid enum values from API documentation
VALID_BODY_ABR_TYPE = [
    "cisco",
    "ibm",
    "standard",
]
VALID_BODY_DEFAULT_INFORMATION_ORIGINATE = [
    "enable",
    "always",
    "disable",
]
VALID_BODY_LOG_NEIGHBOUR_CHANGES = [
    "enable",
    "disable",
]
VALID_BODY_DEFAULT_INFORMATION_METRIC_TYPE = [
    "1",
    "2",
]
VALID_BODY_BFD = [
    "enable",
    "disable",
]
VALID_BODY_RESTART_MODE = [
    "none",
    "graceful-restart",
]
VALID_BODY_RESTART_ON_TOPOLOGY_CHANGE = [
    "enable",
    "disable",
]
VALID_QUERY_ACTION = ["default", "schema"]

# ============================================================================
# GET Validation
# ============================================================================


def validate_router_ospf6_get(
    attr: str | None = None,
    filters: dict[str, Any] | None = None,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate GET request parameters for router/ospf6."""
    # Validate query parameters using central function
    if "action" in params:
        is_valid, error = _validate_query_parameter(
            "action",
            params.get("action"),
            VALID_QUERY_ACTION
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# POST Validation
# ============================================================================


def validate_router_ospf6_post(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate POST request to create new router/ospf6 object."""
    # Step 1: Validate required fields using central function
    is_valid, error = _validate_required_fields(
        payload,
        REQUIRED_FIELDS,
        FIELD_DESCRIPTIONS
    )
    if not is_valid:
        return (False, error)

    # Step 2: Validate enum values using central function
    if "abr-type" in payload:
        is_valid, error = _validate_enum_field(
            "abr-type",
            payload["abr-type"],
            VALID_BODY_ABR_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "default-information-originate" in payload:
        is_valid, error = _validate_enum_field(
            "default-information-originate",
            payload["default-information-originate"],
            VALID_BODY_DEFAULT_INFORMATION_ORIGINATE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "log-neighbour-changes" in payload:
        is_valid, error = _validate_enum_field(
            "log-neighbour-changes",
            payload["log-neighbour-changes"],
            VALID_BODY_LOG_NEIGHBOUR_CHANGES,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "default-information-metric-type" in payload:
        is_valid, error = _validate_enum_field(
            "default-information-metric-type",
            payload["default-information-metric-type"],
            VALID_BODY_DEFAULT_INFORMATION_METRIC_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "bfd" in payload:
        is_valid, error = _validate_enum_field(
            "bfd",
            payload["bfd"],
            VALID_BODY_BFD,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "restart-mode" in payload:
        is_valid, error = _validate_enum_field(
            "restart-mode",
            payload["restart-mode"],
            VALID_BODY_RESTART_MODE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "restart-on-topology-change" in payload:
        is_valid, error = _validate_enum_field(
            "restart-on-topology-change",
            payload["restart-on-topology-change"],
            VALID_BODY_RESTART_ON_TOPOLOGY_CHANGE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# PUT Validation
# ============================================================================


def validate_router_ospf6_put(
    payload: dict,
    **params: Any,
) -> tuple[bool, str | None]:
    """Validate PUT request to update router/ospf6."""
    # Validate enum values using central function
    if "abr-type" in payload:
        is_valid, error = _validate_enum_field(
            "abr-type",
            payload["abr-type"],
            VALID_BODY_ABR_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "default-information-originate" in payload:
        is_valid, error = _validate_enum_field(
            "default-information-originate",
            payload["default-information-originate"],
            VALID_BODY_DEFAULT_INFORMATION_ORIGINATE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "log-neighbour-changes" in payload:
        is_valid, error = _validate_enum_field(
            "log-neighbour-changes",
            payload["log-neighbour-changes"],
            VALID_BODY_LOG_NEIGHBOUR_CHANGES,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "default-information-metric-type" in payload:
        is_valid, error = _validate_enum_field(
            "default-information-metric-type",
            payload["default-information-metric-type"],
            VALID_BODY_DEFAULT_INFORMATION_METRIC_TYPE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "bfd" in payload:
        is_valid, error = _validate_enum_field(
            "bfd",
            payload["bfd"],
            VALID_BODY_BFD,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "restart-mode" in payload:
        is_valid, error = _validate_enum_field(
            "restart-mode",
            payload["restart-mode"],
            VALID_BODY_RESTART_MODE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)
    if "restart-on-topology-change" in payload:
        is_valid, error = _validate_enum_field(
            "restart-on-topology-change",
            payload["restart-on-topology-change"],
            VALID_BODY_RESTART_ON_TOPOLOGY_CHANGE,
            FIELD_DESCRIPTIONS
        )
        if not is_valid:
            return (False, error)

    return (True, None)


# ============================================================================
# Metadata Access Functions
# Imported from central module to avoid duplication across 1,062 files
# Bound to this endpoint's data using functools.partial (saves ~7KB per file)
# ============================================================================

from functools import partial
from hfortix_fortios._helpers.metadata import (
    get_field_description,
    get_field_type,
    get_field_constraints,
    get_field_default,
    get_field_options,
    get_nested_schema,
    get_all_fields,
    get_field_metadata,
    validate_field_value,
)

# Bind module-specific data to central functions using partial application
get_field_description = partial(get_field_description, FIELD_DESCRIPTIONS)
get_field_type = partial(get_field_type, FIELD_TYPES)
get_field_constraints = partial(get_field_constraints, FIELD_CONSTRAINTS)
get_field_default = partial(get_field_default, FIELDS_WITH_DEFAULTS)
get_field_options = partial(get_field_options, globals())
get_nested_schema = partial(get_nested_schema, NESTED_SCHEMAS)
get_all_fields = partial(get_all_fields, FIELD_TYPES)
get_field_metadata = partial(get_field_metadata, FIELD_TYPES, FIELD_DESCRIPTIONS, 
                             FIELD_CONSTRAINTS, FIELDS_WITH_DEFAULTS, REQUIRED_FIELDS,
                             NESTED_SCHEMAS, globals())
validate_field_value = partial(validate_field_value, FIELD_TYPES, FIELD_DESCRIPTIONS,
                               FIELD_CONSTRAINTS, globals())


# ============================================================================
# Schema Information
# Metadata about this endpoint schema
# ============================================================================

SCHEMA_INFO = {
    "endpoint": "router/ospf6",
    "category": "cmdb",
    "api_path": "router/ospf6",
    "help": "Configure IPv6 OSPF.",
    "total_fields": 19,
    "required_fields_count": 0,
    "fields_with_defaults_count": 14,
}


def get_schema_info() -> dict[str, Any]:
    """Get information about this endpoint schema."""
    return SCHEMA_INFO.copy()



"""
Pydantic Models for CMDB - switch_controller/stp_settings

Runtime validation models for switch_controller/stp_settings configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Optional

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class StpSettingsModel(BaseModel):
    """
    Pydantic model for switch_controller/stp_settings configuration.
    
    Configure FortiSwitch spanning tree protocol (STP).
    
    Validation Rules:        - name: max_length=31 pattern=        - revision: min=0 max=65535 pattern=        - hello_time: min=1 max=10 pattern=        - forward_time: min=4 max=30 pattern=        - max_age: min=6 max=40 pattern=        - max_hops: min=1 max=40 pattern=        - pending_timer: min=1 max=15 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str | None = Field(max_length=31, default=None, description="Name of global STP settings configuration.")    
    revision: int | None = Field(ge=0, le=65535, default=0, description="STP revision number (0 - 65535).")    
    hello_time: int | None = Field(ge=1, le=10, default=2, description="Period of time between successive STP frame Bridge Protocol Data Units (BPDUs) sent on a port (1 - 10 sec, default = 2).")    
    forward_time: int | None = Field(ge=4, le=30, default=15, description="Period of time a port is in listening and learning state (4 - 30 sec, default = 15).")    
    max_age: int | None = Field(ge=6, le=40, default=20, description="Maximum time before a bridge port expires its configuration BPDU information (6 - 40 sec, default = 20).")    
    max_hops: int | None = Field(ge=1, le=40, default=20, description="Maximum number of hops between the root bridge and the furthest bridge (1- 40, default = 20).")    
    pending_timer: int | None = Field(ge=1, le=15, default=4, description="Pending time (1 - 15 sec, default = 4).")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "StpSettingsModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "StpSettingsModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:11.502643Z
# ============================================================================


"""
Pydantic Models for CMDB - switch_controller/traffic_policy

Runtime validation models for switch_controller/traffic_policy configuration.
Generated from FortiOS schema version unknown.
"""

from __future__ import annotations

from pydantic import BaseModel, Field
from typing import Any, Literal, Optional

# ============================================================================
# Enum Definitions (for fields with 4+ allowed values)
# ============================================================================


# ============================================================================
# Main Model
# ============================================================================

class TrafficPolicyModel(BaseModel):
    """
    Pydantic model for switch_controller/traffic_policy configuration.
    
    Configure FortiSwitch traffic policy.
    
    Validation Rules:        - name: max_length=63 pattern=        - description: max_length=63 pattern=        - policer_status: pattern=        - guaranteed_bandwidth: min=0 max=524287000 pattern=        - guaranteed_burst: min=0 max=4294967295 pattern=        - maximum_burst: min=0 max=4294967295 pattern=        - type_: pattern=        - cos_queue: min=0 max=7 pattern=    """
    
    class Config:
        """Pydantic model configuration."""
        extra = "allow"  # Allow additional fields from API
        str_strip_whitespace = True
        validate_assignment = True  # Validate on attribute assignment
        use_enum_values = True  # Use enum values instead of names
    
    # ========================================================================
    # Model Fields
    # ========================================================================
    
    name: str = Field(max_length=63, description="Traffic policy name.")    
    description: str | None = Field(max_length=63, default=None, description="Description of the traffic policy.")    
    policer_status: Literal["enable", "disable"] | None = Field(default="enable", description="Enable/disable policer config on the traffic policy.")    
    guaranteed_bandwidth: int | None = Field(ge=0, le=524287000, default=10000, description="Guaranteed bandwidth in kbps (max value = 524287000).")    
    guaranteed_burst: int | None = Field(ge=0, le=4294967295, default=45000, description="Guaranteed burst size in bytes (max value = 4294967295).")    
    maximum_burst: int | None = Field(ge=0, le=4294967295, default=67500, description="Maximum burst size in bytes (max value = 4294967295).")    
    type_: Literal["ingress", "egress"] | None = Field(default="ingress", serialization_alias="type", description="Configure type of policy(ingress/egress).")    
    cos_queue: int | None = Field(ge=0, le=7, default=None, description="COS queue(0 - 7), or unset to disable.")    
    # ========================================================================
    # Custom Validators
    # ========================================================================
    
    # ========================================================================
    # Helper Methods
    # ========================================================================
    
    def to_fortios_dict(self) -> dict[str, Any]:
        """
        Convert model to FortiOS API payload format.
        
        Returns:
            Dict suitable for POST/PUT operations
        """
        # Export with exclude_none to avoid sending null values
        return self.model_dump(exclude_none=True, by_alias=True)
    
    @classmethod
    def from_fortios_response(cls, data: dict[str, Any]) -> "TrafficPolicyModel":
        """
        Create model instance from FortiOS API response.
        
        Args:
            data: Response data from API
            
        Returns:
            Validated model instance
        """
        return cls(**data)

# ============================================================================
# Type Aliases for Convenience
# ============================================================================

Dict = dict[str, Any]  # For backward compatibility

# ============================================================================
# Module Exports
# ============================================================================

__all__ = [
    "TrafficPolicyModel",]


# ============================================================================
# Generated by hfortix generator v0.6.0
# Schema: 1.7.0
# Generated: 2026-01-17T17:56:13.204380Z
# ============================================================================


""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: wireless_controller/qos_profile
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
    overload,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class QosProfileDscpwmmvoItem:
    """Nested item for dscp-wmm-vo field - supports attribute access."""
    id: int


class QosProfileDscpwmmviItem:
    """Nested item for dscp-wmm-vi field - supports attribute access."""
    id: int


class QosProfileDscpwmmbeItem:
    """Nested item for dscp-wmm-be field - supports attribute access."""
    id: int


class QosProfileDscpwmmbkItem:
    """Nested item for dscp-wmm-bk field - supports attribute access."""
    id: int


class QosProfilePayload(TypedDict, total=False):
    """Payload type for QosProfile operations."""
    name: str
    comment: str
    uplink: int
    downlink: int
    uplink_sta: int
    downlink_sta: int
    burst: Literal["enable", "disable"]
    wmm: Literal["enable", "disable"]
    wmm_uapsd: Literal["enable", "disable"]
    call_admission_control: Literal["enable", "disable"]
    call_capacity: int
    bandwidth_admission_control: Literal["enable", "disable"]
    bandwidth_capacity: int
    dscp_wmm_mapping: Literal["enable", "disable"]
    dscp_wmm_vo: str | list[str] | list[dict[str, Any]] | list[QosProfileDscpwmmvoItem]
    dscp_wmm_vi: str | list[str] | list[dict[str, Any]] | list[QosProfileDscpwmmviItem]
    dscp_wmm_be: str | list[str] | list[dict[str, Any]] | list[QosProfileDscpwmmbeItem]
    dscp_wmm_bk: str | list[str] | list[dict[str, Any]] | list[QosProfileDscpwmmbkItem]
    wmm_dscp_marking: Literal["enable", "disable"]
    wmm_vo_dscp: int
    wmm_vi_dscp: int
    wmm_be_dscp: int
    wmm_bk_dscp: int


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class QosProfileResponse(TypedDict, total=False):
    """Response type for QosProfile - use with .dict property for typed dict access."""
    name: str
    comment: str
    uplink: int
    downlink: int
    uplink_sta: int
    downlink_sta: int
    burst: Literal["enable", "disable"]
    wmm: Literal["enable", "disable"]
    wmm_uapsd: Literal["enable", "disable"]
    call_admission_control: Literal["enable", "disable"]
    call_capacity: int
    bandwidth_admission_control: Literal["enable", "disable"]
    bandwidth_capacity: int
    dscp_wmm_mapping: Literal["enable", "disable"]
    dscp_wmm_vo: list[QosProfileDscpwmmvoItem]
    dscp_wmm_vi: list[QosProfileDscpwmmviItem]
    dscp_wmm_be: list[QosProfileDscpwmmbeItem]
    dscp_wmm_bk: list[QosProfileDscpwmmbkItem]
    wmm_dscp_marking: Literal["enable", "disable"]
    wmm_vo_dscp: int
    wmm_vi_dscp: int
    wmm_be_dscp: int
    wmm_bk_dscp: int


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class QosProfileObject(FortiObject):
    """Typed FortiObject for QosProfile with field access."""
    name: str
    comment: str
    uplink: int
    downlink: int
    uplink_sta: int
    downlink_sta: int
    burst: Literal["enable", "disable"]
    wmm: Literal["enable", "disable"]
    wmm_uapsd: Literal["enable", "disable"]
    call_admission_control: Literal["enable", "disable"]
    call_capacity: int
    bandwidth_admission_control: Literal["enable", "disable"]
    bandwidth_capacity: int
    dscp_wmm_mapping: Literal["enable", "disable"]
    dscp_wmm_vo: list[QosProfileDscpwmmvoItem]
    dscp_wmm_vi: list[QosProfileDscpwmmviItem]
    dscp_wmm_be: list[QosProfileDscpwmmbeItem]
    dscp_wmm_bk: list[QosProfileDscpwmmbkItem]
    wmm_dscp_marking: Literal["enable", "disable"]
    wmm_vo_dscp: int
    wmm_vi_dscp: int
    wmm_be_dscp: int
    wmm_bk_dscp: int


# ================================================================
# Main Endpoint Class
# ================================================================

class QosProfile:
    """
    
    Endpoint: wireless_controller/qos_profile
    Category: cmdb
    MKey: name
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    mkey: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # CMDB with mkey - overloads for single vs list returns
    @overload
    def get(
        self,
        name: str,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> QosProfileObject: ...
    
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObjectList[QosProfileObject]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...

    # ================================================================
    # POST Method
    # ================================================================
    
    def post(
        self,
        payload_dict: QosProfilePayload | None = ...,
        name: str | None = ...,
        comment: str | None = ...,
        uplink: int | None = ...,
        downlink: int | None = ...,
        uplink_sta: int | None = ...,
        downlink_sta: int | None = ...,
        burst: Literal["enable", "disable"] | None = ...,
        wmm: Literal["enable", "disable"] | None = ...,
        wmm_uapsd: Literal["enable", "disable"] | None = ...,
        call_admission_control: Literal["enable", "disable"] | None = ...,
        call_capacity: int | None = ...,
        bandwidth_admission_control: Literal["enable", "disable"] | None = ...,
        bandwidth_capacity: int | None = ...,
        dscp_wmm_mapping: Literal["enable", "disable"] | None = ...,
        dscp_wmm_vo: str | list[str] | list[dict[str, Any]] | list[QosProfileDscpwmmvoItem] | None = ...,
        dscp_wmm_vi: str | list[str] | list[dict[str, Any]] | list[QosProfileDscpwmmviItem] | None = ...,
        dscp_wmm_be: str | list[str] | list[dict[str, Any]] | list[QosProfileDscpwmmbeItem] | None = ...,
        dscp_wmm_bk: str | list[str] | list[dict[str, Any]] | list[QosProfileDscpwmmbkItem] | None = ...,
        wmm_dscp_marking: Literal["enable", "disable"] | None = ...,
        wmm_vo_dscp: int | None = ...,
        wmm_vi_dscp: int | None = ...,
        wmm_be_dscp: int | None = ...,
        wmm_bk_dscp: int | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> QosProfileObject: ...

    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: QosProfilePayload | None = ...,
        name: str | None = ...,
        comment: str | None = ...,
        uplink: int | None = ...,
        downlink: int | None = ...,
        uplink_sta: int | None = ...,
        downlink_sta: int | None = ...,
        burst: Literal["enable", "disable"] | None = ...,
        wmm: Literal["enable", "disable"] | None = ...,
        wmm_uapsd: Literal["enable", "disable"] | None = ...,
        call_admission_control: Literal["enable", "disable"] | None = ...,
        call_capacity: int | None = ...,
        bandwidth_admission_control: Literal["enable", "disable"] | None = ...,
        bandwidth_capacity: int | None = ...,
        dscp_wmm_mapping: Literal["enable", "disable"] | None = ...,
        dscp_wmm_vo: str | list[str] | list[dict[str, Any]] | list[QosProfileDscpwmmvoItem] | None = ...,
        dscp_wmm_vi: str | list[str] | list[dict[str, Any]] | list[QosProfileDscpwmmviItem] | None = ...,
        dscp_wmm_be: str | list[str] | list[dict[str, Any]] | list[QosProfileDscpwmmbeItem] | None = ...,
        dscp_wmm_bk: str | list[str] | list[dict[str, Any]] | list[QosProfileDscpwmmbkItem] | None = ...,
        wmm_dscp_marking: Literal["enable", "disable"] | None = ...,
        wmm_vo_dscp: int | None = ...,
        wmm_vi_dscp: int | None = ...,
        wmm_be_dscp: int | None = ...,
        wmm_bk_dscp: int | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> QosProfileObject: ...

    # ================================================================
    # DELETE Method
    # ================================================================
    
    def delete(
        self,
        name: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject: ...

    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        name: str,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: QosProfilePayload | None = ...,
        name: str | None = ...,
        comment: str | None = ...,
        uplink: int | None = ...,
        downlink: int | None = ...,
        uplink_sta: int | None = ...,
        downlink_sta: int | None = ...,
        burst: Literal["enable", "disable"] | None = ...,
        wmm: Literal["enable", "disable"] | None = ...,
        wmm_uapsd: Literal["enable", "disable"] | None = ...,
        call_admission_control: Literal["enable", "disable"] | None = ...,
        call_capacity: int | None = ...,
        bandwidth_admission_control: Literal["enable", "disable"] | None = ...,
        bandwidth_capacity: int | None = ...,
        dscp_wmm_mapping: Literal["enable", "disable"] | None = ...,
        dscp_wmm_vo: str | list[str] | list[dict[str, Any]] | list[QosProfileDscpwmmvoItem] | None = ...,
        dscp_wmm_vi: str | list[str] | list[dict[str, Any]] | list[QosProfileDscpwmmviItem] | None = ...,
        dscp_wmm_be: str | list[str] | list[dict[str, Any]] | list[QosProfileDscpwmmbeItem] | None = ...,
        dscp_wmm_bk: str | list[str] | list[dict[str, Any]] | list[QosProfileDscpwmmbkItem] | None = ...,
        wmm_dscp_marking: Literal["enable", "disable"] | None = ...,
        wmm_vo_dscp: int | None = ...,
        wmm_vi_dscp: int | None = ...,
        wmm_be_dscp: int | None = ...,
        wmm_bk_dscp: int | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject: ...
    
    @staticmethod
    def schema() -> FortiObject: ...


__all__ = [
    "QosProfile",
    "QosProfilePayload",
    "QosProfileResponse",
    "QosProfileObject",
]


from typing import Any, Literal

# Enum type aliases for validation
VALID_BODY_DHCP_IPSEC: Literal["enable", "disable"]
VALID_BODY_PROPOSAL: Literal["null-md5", "null-sha1", "null-sha256", "null-sha384", "null-sha512", "des-null", "des-md5", "des-sha1", "des-sha256", "des-sha384", "des-sha512", "3des-null", "3des-md5", "3des-sha1", "3des-sha256", "3des-sha384", "3des-sha512", "aes128-null", "aes128-md5", "aes128-sha1", "aes128-sha256", "aes128-sha384", "aes128-sha512", "aes128gcm", "aes192-null", "aes192-md5", "aes192-sha1", "aes192-sha256", "aes192-sha384", "aes192-sha512", "aes256-null", "aes256-md5", "aes256-sha1", "aes256-sha256", "aes256-sha384", "aes256-sha512", "aes256gcm", "chacha20poly1305", "aria128-null", "aria128-md5", "aria128-sha1", "aria128-sha256", "aria128-sha384", "aria128-sha512", "aria192-null", "aria192-md5", "aria192-sha1", "aria192-sha256", "aria192-sha384", "aria192-sha512", "aria256-null", "aria256-md5", "aria256-sha1", "aria256-sha256", "aria256-sha384", "aria256-sha512", "seed-null", "seed-md5", "seed-sha1", "seed-sha256", "seed-sha384", "seed-sha512"]
VALID_BODY_PFS: Literal["enable", "disable"]
VALID_BODY_DHGRP: Literal["1", "2", "5", "14", "15", "16", "17", "18", "19", "20", "21", "27", "28", "29", "30", "31", "32"]
VALID_BODY_ADDKE1: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"]
VALID_BODY_ADDKE2: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"]
VALID_BODY_ADDKE3: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"]
VALID_BODY_ADDKE4: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"]
VALID_BODY_ADDKE5: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"]
VALID_BODY_ADDKE6: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"]
VALID_BODY_ADDKE7: Literal["0", "35", "36", "37", "1080", "1081", "1082", "1083", "1084", "1085", "1089", "1090", "1091", "1092", "1093", "1094"]
VALID_BODY_REPLAY: Literal["enable", "disable"]
VALID_BODY_KEEPALIVE: Literal["enable", "disable"]
VALID_BODY_AUTO_NEGOTIATE: Literal["enable", "disable"]
VALID_BODY_ADD_ROUTE: Literal["phase1", "enable", "disable"]
VALID_BODY_INBOUND_DSCP_COPY: Literal["phase1", "enable", "disable"]
VALID_BODY_AUTO_DISCOVERY_SENDER: Literal["phase1", "enable", "disable"]
VALID_BODY_AUTO_DISCOVERY_FORWARDER: Literal["phase1", "enable", "disable"]
VALID_BODY_KEYLIFE_TYPE: Literal["seconds", "kbs", "both"]
VALID_BODY_SINGLE_SOURCE: Literal["enable", "disable"]
VALID_BODY_ROUTE_OVERLAP: Literal["use-old", "use-new", "allow"]
VALID_BODY_ENCAPSULATION: Literal["tunnel-mode", "transport-mode"]
VALID_BODY_L2TP: Literal["enable", "disable"]
VALID_BODY_INITIATOR_TS_NARROW: Literal["enable", "disable"]
VALID_BODY_DIFFSERV: Literal["enable", "disable"]
VALID_BODY_SRC_ADDR_TYPE: Literal["subnet", "range", "ip", "name", "subnet6", "range6", "ip6", "name6"]
VALID_BODY_DST_ADDR_TYPE: Literal["subnet", "range", "ip", "name", "subnet6", "range6", "ip6", "name6"]

# Metadata dictionaries
FIELD_TYPES: dict[str, str]
FIELD_DESCRIPTIONS: dict[str, str]
FIELD_CONSTRAINTS: dict[str, dict[str, Any]]
NESTED_SCHEMAS: dict[str, dict[str, Any]]
FIELDS_WITH_DEFAULTS: dict[str, Any]
DEPRECATED_FIELDS: dict[str, dict[str, str]]
REQUIRED_FIELDS: list[str]

# Helper functions
def get_field_type(field_name: str) -> str | None: ...
def get_field_description(field_name: str) -> str | None: ...
def get_field_default(field_name: str) -> Any: ...
def get_field_constraints(field_name: str) -> dict[str, Any]: ...
def get_nested_schema(field_name: str) -> dict[str, Any] | None: ...
def get_field_metadata(field_name: str) -> dict[str, Any]: ...
def validate_field_value(field_name: str, value: Any) -> bool: ...
def get_all_fields() -> list[str]: ...
def get_required_fields() -> list[str]: ...
def get_schema_info() -> dict[str, Any]: ...


__all__ = [
    "VALID_BODY_DHCP_IPSEC",
    "VALID_BODY_PROPOSAL",
    "VALID_BODY_PFS",
    "VALID_BODY_DHGRP",
    "VALID_BODY_ADDKE1",
    "VALID_BODY_ADDKE2",
    "VALID_BODY_ADDKE3",
    "VALID_BODY_ADDKE4",
    "VALID_BODY_ADDKE5",
    "VALID_BODY_ADDKE6",
    "VALID_BODY_ADDKE7",
    "VALID_BODY_REPLAY",
    "VALID_BODY_KEEPALIVE",
    "VALID_BODY_AUTO_NEGOTIATE",
    "VALID_BODY_ADD_ROUTE",
    "VALID_BODY_INBOUND_DSCP_COPY",
    "VALID_BODY_AUTO_DISCOVERY_SENDER",
    "VALID_BODY_AUTO_DISCOVERY_FORWARDER",
    "VALID_BODY_KEYLIFE_TYPE",
    "VALID_BODY_SINGLE_SOURCE",
    "VALID_BODY_ROUTE_OVERLAP",
    "VALID_BODY_ENCAPSULATION",
    "VALID_BODY_L2TP",
    "VALID_BODY_INITIATOR_TS_NARROW",
    "VALID_BODY_DIFFSERV",
    "VALID_BODY_SRC_ADDR_TYPE",
    "VALID_BODY_DST_ADDR_TYPE",
    "FIELD_TYPES",
    "FIELD_DESCRIPTIONS",
    "FIELD_CONSTRAINTS",
    "NESTED_SCHEMAS",
    "FIELDS_WITH_DEFAULTS",
    "DEPRECATED_FIELDS",
    "REQUIRED_FIELDS",
    "get_field_type",
    "get_field_description",
    "get_field_default",
    "get_field_constraints",
    "get_nested_schema",
    "get_field_metadata",
    "validate_field_value",
    "get_all_fields",
    "get_required_fields",
    "get_schema_info",
]


"""FortiOS CMDB - Hotspot20 category"""

from .anqp_3gpp_cellular import Anqp3gppCellular
from .anqp_ip_address_type import AnqpIpAddressType
from .anqp_nai_realm import AnqpNaiRealm
from .anqp_network_auth_type import AnqpNetworkAuthType
from .anqp_roaming_consortium import AnqpRoamingConsortium
from .anqp_venue_name import AnqpVenueName
from .anqp_venue_url import AnqpVenueUrl
from .h2qp_advice_of_charge import H2qpAdviceOfCharge
from .h2qp_conn_capability import H2qpConnCapability
from .h2qp_operator_name import H2qpOperatorName
from .h2qp_osu_provider import H2qpOsuProvider
from .h2qp_osu_provider_nai import H2qpOsuProviderNai
from .h2qp_terms_and_conditions import H2qpTermsAndConditions
from .h2qp_wan_metric import H2qpWanMetric
from .hs_profile import HsProfile
from .icon import Icon
from .qos_map import QosMap

__all__ = [
    "Anqp3gppCellular",
    "AnqpIpAddressType",
    "AnqpNaiRealm",
    "AnqpNetworkAuthType",
    "AnqpRoamingConsortium",
    "AnqpVenueName",
    "AnqpVenueUrl",
    "H2qpAdviceOfCharge",
    "H2qpConnCapability",
    "H2qpOperatorName",
    "H2qpOsuProvider",
    "H2qpOsuProviderNai",
    "H2qpTermsAndConditions",
    "H2qpWanMetric",
    "Hotspot20",
    "HsProfile",
    "Icon",
    "QosMap",
]


class Hotspot20:
    """Hotspot20 endpoints wrapper for CMDB API."""

    def __init__(self, client):
        """Hotspot20 endpoints.
        
        Args:
            client: HTTP client instance for API communication
        """
        self.anqp_3gpp_cellular = Anqp3gppCellular(client)
        self.anqp_ip_address_type = AnqpIpAddressType(client)
        self.anqp_nai_realm = AnqpNaiRealm(client)
        self.anqp_network_auth_type = AnqpNetworkAuthType(client)
        self.anqp_roaming_consortium = AnqpRoamingConsortium(client)
        self.anqp_venue_name = AnqpVenueName(client)
        self.anqp_venue_url = AnqpVenueUrl(client)
        self.h2qp_advice_of_charge = H2qpAdviceOfCharge(client)
        self.h2qp_conn_capability = H2qpConnCapability(client)
        self.h2qp_operator_name = H2qpOperatorName(client)
        self.h2qp_osu_provider = H2qpOsuProvider(client)
        self.h2qp_osu_provider_nai = H2qpOsuProviderNai(client)
        self.h2qp_terms_and_conditions = H2qpTermsAndConditions(client)
        self.h2qp_wan_metric = H2qpWanMetric(client)
        self.hs_profile = HsProfile(client)
        self.icon = Icon(client)
        self.qos_map = QosMap(client)



""" - Type Stubs

Auto-generated stub file for type checking and IDE support.

Endpoint: firewall/policy
Category: cmdb
"""

from __future__ import annotations

from typing import (
    Any,
    ClassVar,
    Literal,
    TypedDict,
    overload,
)

from hfortix_fortios.models import (
    FortiObject,
    FortiObjectList,
)


# ================================================================
# TypedDict Payloads
# ================================================================

class PolicySrcintfItem:
    """Nested item for srcintf field - supports attribute access."""
    name: str


class PolicyDstintfItem:
    """Nested item for dstintf field - supports attribute access."""
    name: str


class PolicySrcaddrItem:
    """Nested item for srcaddr field - supports attribute access."""
    name: str


class PolicyDstaddrItem:
    """Nested item for dstaddr field - supports attribute access."""
    name: str


class PolicySrcaddr6Item:
    """Nested item for srcaddr6 field - supports attribute access."""
    name: str


class PolicyDstaddr6Item:
    """Nested item for dstaddr6 field - supports attribute access."""
    name: str


class PolicyZtnaemstagItem:
    """Nested item for ztna-ems-tag field - supports attribute access."""
    name: str


class PolicyZtnaemstagsecondaryItem:
    """Nested item for ztna-ems-tag-secondary field - supports attribute access."""
    name: str


class PolicyZtnageotagItem:
    """Nested item for ztna-geo-tag field - supports attribute access."""
    name: str


class PolicyInternetservicenameItem:
    """Nested item for internet-service-name field - supports attribute access."""
    name: str


class PolicyInternetservicegroupItem:
    """Nested item for internet-service-group field - supports attribute access."""
    name: str


class PolicyInternetservicecustomItem:
    """Nested item for internet-service-custom field - supports attribute access."""
    name: str


class PolicyNetworkservicedynamicItem:
    """Nested item for network-service-dynamic field - supports attribute access."""
    name: str


class PolicyInternetservicecustomgroupItem:
    """Nested item for internet-service-custom-group field - supports attribute access."""
    name: str


class PolicyInternetservicesrcnameItem:
    """Nested item for internet-service-src-name field - supports attribute access."""
    name: str


class PolicyInternetservicesrcgroupItem:
    """Nested item for internet-service-src-group field - supports attribute access."""
    name: str


class PolicyInternetservicesrccustomItem:
    """Nested item for internet-service-src-custom field - supports attribute access."""
    name: str


class PolicyNetworkservicesrcdynamicItem:
    """Nested item for network-service-src-dynamic field - supports attribute access."""
    name: str


class PolicyInternetservicesrccustomgroupItem:
    """Nested item for internet-service-src-custom-group field - supports attribute access."""
    name: str


class PolicySrcvendormacItem:
    """Nested item for src-vendor-mac field - supports attribute access."""
    id: int


class PolicyInternetservice6nameItem:
    """Nested item for internet-service6-name field - supports attribute access."""
    name: str


class PolicyInternetservice6groupItem:
    """Nested item for internet-service6-group field - supports attribute access."""
    name: str


class PolicyInternetservice6customItem:
    """Nested item for internet-service6-custom field - supports attribute access."""
    name: str


class PolicyInternetservice6customgroupItem:
    """Nested item for internet-service6-custom-group field - supports attribute access."""
    name: str


class PolicyInternetservice6srcnameItem:
    """Nested item for internet-service6-src-name field - supports attribute access."""
    name: str


class PolicyInternetservice6srcgroupItem:
    """Nested item for internet-service6-src-group field - supports attribute access."""
    name: str


class PolicyInternetservice6srccustomItem:
    """Nested item for internet-service6-src-custom field - supports attribute access."""
    name: str


class PolicyInternetservice6srccustomgroupItem:
    """Nested item for internet-service6-src-custom-group field - supports attribute access."""
    name: str


class PolicyRtpaddrItem:
    """Nested item for rtp-addr field - supports attribute access."""
    name: str


class PolicyServiceItem:
    """Nested item for service field - supports attribute access."""
    name: str


class PolicyPcppoolnameItem:
    """Nested item for pcp-poolname field - supports attribute access."""
    name: str


class PolicyPoolnameItem:
    """Nested item for poolname field - supports attribute access."""
    name: str


class PolicyPoolname6Item:
    """Nested item for poolname6 field - supports attribute access."""
    name: str


class PolicyNtlmenabledbrowsersItem:
    """Nested item for ntlm-enabled-browsers field - supports attribute access."""
    user_agent_string: str


class PolicyGroupsItem:
    """Nested item for groups field - supports attribute access."""
    name: str


class PolicyUsersItem:
    """Nested item for users field - supports attribute access."""
    name: str


class PolicyFssogroupsItem:
    """Nested item for fsso-groups field - supports attribute access."""
    name: str


class PolicyCustomlogfieldsItem:
    """Nested item for custom-log-fields field - supports attribute access."""
    field_id: str


class PolicySgtItem:
    """Nested item for sgt field - supports attribute access."""
    id: int


class PolicyInternetservicefortiguardItem:
    """Nested item for internet-service-fortiguard field - supports attribute access."""
    name: str


class PolicyInternetservicesrcfortiguardItem:
    """Nested item for internet-service-src-fortiguard field - supports attribute access."""
    name: str


class PolicyInternetservice6fortiguardItem:
    """Nested item for internet-service6-fortiguard field - supports attribute access."""
    name: str


class PolicyInternetservice6srcfortiguardItem:
    """Nested item for internet-service6-src-fortiguard field - supports attribute access."""
    name: str


class PolicyPayload(TypedDict, total=False):
    """Payload type for Policy operations."""
    policyid: int
    status: Literal["enable", "disable"]
    name: str
    uuid: str
    srcintf: str | list[str] | list[dict[str, Any]] | list[PolicySrcintfItem]
    dstintf: str | list[str] | list[dict[str, Any]] | list[PolicyDstintfItem]
    action: Literal["accept", "deny", "ipsec"]
    nat64: Literal["enable", "disable"]
    nat46: Literal["enable", "disable"]
    ztna_status: Literal["enable", "disable"]
    ztna_device_ownership: Literal["enable", "disable"]
    srcaddr: str | list[str] | list[dict[str, Any]] | list[PolicySrcaddrItem]
    dstaddr: str | list[str] | list[dict[str, Any]] | list[PolicyDstaddrItem]
    srcaddr6: str | list[str] | list[dict[str, Any]] | list[PolicySrcaddr6Item]
    dstaddr6: str | list[str] | list[dict[str, Any]] | list[PolicyDstaddr6Item]
    ztna_ems_tag: str | list[str] | list[dict[str, Any]] | list[PolicyZtnaemstagItem]
    ztna_ems_tag_secondary: str | list[str] | list[dict[str, Any]] | list[PolicyZtnaemstagsecondaryItem]
    ztna_tags_match_logic: Literal["or", "and"]
    ztna_geo_tag: str | list[str] | list[dict[str, Any]] | list[PolicyZtnageotagItem]
    internet_service: Literal["enable", "disable"]
    internet_service_name: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicenameItem]
    internet_service_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicegroupItem]
    internet_service_custom: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicecustomItem]
    network_service_dynamic: str | list[str] | list[dict[str, Any]] | list[PolicyNetworkservicedynamicItem]
    internet_service_custom_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicecustomgroupItem]
    internet_service_src: Literal["enable", "disable"]
    internet_service_src_name: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicesrcnameItem]
    internet_service_src_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicesrcgroupItem]
    internet_service_src_custom: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicesrccustomItem]
    network_service_src_dynamic: str | list[str] | list[dict[str, Any]] | list[PolicyNetworkservicesrcdynamicItem]
    internet_service_src_custom_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicesrccustomgroupItem]
    reputation_minimum: int
    reputation_direction: Literal["source", "destination"]
    src_vendor_mac: str | list[str] | list[dict[str, Any]] | list[PolicySrcvendormacItem]
    internet_service6: Literal["enable", "disable"]
    internet_service6_name: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6nameItem]
    internet_service6_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6groupItem]
    internet_service6_custom: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6customItem]
    internet_service6_custom_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6customgroupItem]
    internet_service6_src: Literal["enable", "disable"]
    internet_service6_src_name: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6srcnameItem]
    internet_service6_src_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6srcgroupItem]
    internet_service6_src_custom: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6srccustomItem]
    internet_service6_src_custom_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6srccustomgroupItem]
    reputation_minimum6: int
    reputation_direction6: Literal["source", "destination"]
    rtp_nat: Literal["disable", "enable"]
    rtp_addr: str | list[str] | list[dict[str, Any]] | list[PolicyRtpaddrItem]
    send_deny_packet: Literal["disable", "enable"]
    firewall_session_dirty: Literal["check-all", "check-new"]
    schedule: str
    schedule_timeout: Literal["enable", "disable"]
    policy_expiry: Literal["enable", "disable"]
    policy_expiry_date: str
    policy_expiry_date_utc: str
    service: str | list[str] | list[dict[str, Any]] | list[PolicyServiceItem]
    tos_mask: str
    tos: str
    tos_negate: Literal["enable", "disable"]
    anti_replay: Literal["enable", "disable"]
    tcp_session_without_syn: Literal["all", "data-only", "disable"]
    geoip_anycast: Literal["enable", "disable"]
    geoip_match: Literal["physical-location", "registered-location"]
    dynamic_shaping: Literal["enable", "disable"]
    passive_wan_health_measurement: Literal["enable", "disable"]
    app_monitor: Literal["enable", "disable"]
    utm_status: Literal["enable", "disable"]
    inspection_mode: Literal["proxy", "flow"]
    http_policy_redirect: Literal["enable", "disable", "legacy"]
    ssh_policy_redirect: Literal["enable", "disable"]
    ztna_policy_redirect: Literal["enable", "disable"]
    webproxy_profile: str
    profile_type: Literal["single", "group"]
    profile_group: str
    profile_protocol_options: str
    ssl_ssh_profile: str
    av_profile: str
    webfilter_profile: str
    dnsfilter_profile: str
    emailfilter_profile: str
    dlp_profile: str
    file_filter_profile: str
    ips_sensor: str
    application_list: str
    voip_profile: str
    ips_voip_filter: str
    sctp_filter_profile: str
    diameter_filter_profile: str
    virtual_patch_profile: str
    icap_profile: str
    videofilter_profile: str
    waf_profile: str
    ssh_filter_profile: str
    casb_profile: str
    logtraffic: Literal["all", "utm", "disable"]
    logtraffic_start: Literal["enable", "disable"]
    log_http_transaction: Literal["enable", "disable"]
    capture_packet: Literal["enable", "disable"]
    auto_asic_offload: Literal["enable", "disable"]
    wanopt: Literal["enable", "disable"]
    wanopt_detection: Literal["active", "passive", "off"]
    wanopt_passive_opt: Literal["default", "transparent", "non-transparent"]
    wanopt_profile: str
    wanopt_peer: str
    webcache: Literal["enable", "disable"]
    webcache_https: Literal["disable", "enable"]
    webproxy_forward_server: str
    traffic_shaper: str
    traffic_shaper_reverse: str
    per_ip_shaper: str
    nat: Literal["enable", "disable"]
    pcp_outbound: Literal["enable", "disable"]
    pcp_inbound: Literal["enable", "disable"]
    pcp_poolname: str | list[str] | list[dict[str, Any]] | list[PolicyPcppoolnameItem]
    permit_any_host: Literal["enable", "disable"]
    permit_stun_host: Literal["enable", "disable"]
    fixedport: Literal["enable", "disable"]
    port_preserve: Literal["enable", "disable"]
    port_random: Literal["enable", "disable"]
    ippool: Literal["enable", "disable"]
    poolname: str | list[str] | list[dict[str, Any]] | list[PolicyPoolnameItem]
    poolname6: str | list[str] | list[dict[str, Any]] | list[PolicyPoolname6Item]
    session_ttl: str
    vlan_cos_fwd: int
    vlan_cos_rev: int
    inbound: Literal["enable", "disable"]
    outbound: Literal["enable", "disable"]
    natinbound: Literal["enable", "disable"]
    natoutbound: Literal["enable", "disable"]
    fec: Literal["enable", "disable"]
    wccp: Literal["enable", "disable"]
    ntlm: Literal["enable", "disable"]
    ntlm_guest: Literal["enable", "disable"]
    ntlm_enabled_browsers: str | list[str] | list[dict[str, Any]] | list[PolicyNtlmenabledbrowsersItem]
    fsso_agent_for_ntlm: str
    groups: str | list[str] | list[dict[str, Any]] | list[PolicyGroupsItem]
    users: str | list[str] | list[dict[str, Any]] | list[PolicyUsersItem]
    fsso_groups: str | list[str] | list[dict[str, Any]] | list[PolicyFssogroupsItem]
    auth_path: Literal["enable", "disable"]
    disclaimer: Literal["enable", "disable"]
    email_collect: Literal["enable", "disable"]
    vpntunnel: str
    natip: str
    match_vip: Literal["enable", "disable"]
    match_vip_only: Literal["enable", "disable"]
    diffserv_copy: Literal["enable", "disable"]
    diffserv_forward: Literal["enable", "disable"]
    diffserv_reverse: Literal["enable", "disable"]
    diffservcode_forward: str
    diffservcode_rev: str
    tcp_mss_sender: int
    tcp_mss_receiver: int
    comments: str
    auth_cert: str
    auth_redirect_addr: str
    redirect_url: str
    identity_based_route: str
    block_notification: Literal["enable", "disable"]
    custom_log_fields: str | list[str] | list[dict[str, Any]] | list[PolicyCustomlogfieldsItem]
    replacemsg_override_group: str
    srcaddr_negate: Literal["enable", "disable"]
    srcaddr6_negate: Literal["enable", "disable"]
    dstaddr_negate: Literal["enable", "disable"]
    dstaddr6_negate: Literal["enable", "disable"]
    ztna_ems_tag_negate: Literal["enable", "disable"]
    service_negate: Literal["enable", "disable"]
    internet_service_negate: Literal["enable", "disable"]
    internet_service_src_negate: Literal["enable", "disable"]
    internet_service6_negate: Literal["enable", "disable"]
    internet_service6_src_negate: Literal["enable", "disable"]
    timeout_send_rst: Literal["enable", "disable"]
    captive_portal_exempt: Literal["enable", "disable"]
    decrypted_traffic_mirror: str
    dsri: Literal["enable", "disable"]
    radius_mac_auth_bypass: Literal["enable", "disable"]
    radius_ip_auth_bypass: Literal["enable", "disable"]
    delay_tcp_npu_session: Literal["enable", "disable"]
    vlan_filter: str
    sgt_check: Literal["enable", "disable"]
    sgt: str | list[str] | list[dict[str, Any]] | list[PolicySgtItem]
    internet_service_fortiguard: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicefortiguardItem]
    internet_service_src_fortiguard: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicesrcfortiguardItem]
    internet_service6_fortiguard: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6fortiguardItem]
    internet_service6_src_fortiguard: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6srcfortiguardItem]


# ================================================================
# Response Types (TypedDict for dict-style access)
# ================================================================

class PolicyResponse(TypedDict, total=False):
    """Response type for Policy - use with .dict property for typed dict access."""
    policyid: int
    status: Literal["enable", "disable"]
    name: str
    uuid: str
    srcintf: list[PolicySrcintfItem]
    dstintf: list[PolicyDstintfItem]
    action: Literal["accept", "deny", "ipsec"]
    nat64: Literal["enable", "disable"]
    nat46: Literal["enable", "disable"]
    ztna_status: Literal["enable", "disable"]
    ztna_device_ownership: Literal["enable", "disable"]
    srcaddr: list[PolicySrcaddrItem]
    dstaddr: list[PolicyDstaddrItem]
    srcaddr6: list[PolicySrcaddr6Item]
    dstaddr6: list[PolicyDstaddr6Item]
    ztna_ems_tag: list[PolicyZtnaemstagItem]
    ztna_ems_tag_secondary: list[PolicyZtnaemstagsecondaryItem]
    ztna_tags_match_logic: Literal["or", "and"]
    ztna_geo_tag: list[PolicyZtnageotagItem]
    internet_service: Literal["enable", "disable"]
    internet_service_name: list[PolicyInternetservicenameItem]
    internet_service_group: list[PolicyInternetservicegroupItem]
    internet_service_custom: list[PolicyInternetservicecustomItem]
    network_service_dynamic: list[PolicyNetworkservicedynamicItem]
    internet_service_custom_group: list[PolicyInternetservicecustomgroupItem]
    internet_service_src: Literal["enable", "disable"]
    internet_service_src_name: list[PolicyInternetservicesrcnameItem]
    internet_service_src_group: list[PolicyInternetservicesrcgroupItem]
    internet_service_src_custom: list[PolicyInternetservicesrccustomItem]
    network_service_src_dynamic: list[PolicyNetworkservicesrcdynamicItem]
    internet_service_src_custom_group: list[PolicyInternetservicesrccustomgroupItem]
    reputation_minimum: int
    reputation_direction: Literal["source", "destination"]
    src_vendor_mac: list[PolicySrcvendormacItem]
    internet_service6: Literal["enable", "disable"]
    internet_service6_name: list[PolicyInternetservice6nameItem]
    internet_service6_group: list[PolicyInternetservice6groupItem]
    internet_service6_custom: list[PolicyInternetservice6customItem]
    internet_service6_custom_group: list[PolicyInternetservice6customgroupItem]
    internet_service6_src: Literal["enable", "disable"]
    internet_service6_src_name: list[PolicyInternetservice6srcnameItem]
    internet_service6_src_group: list[PolicyInternetservice6srcgroupItem]
    internet_service6_src_custom: list[PolicyInternetservice6srccustomItem]
    internet_service6_src_custom_group: list[PolicyInternetservice6srccustomgroupItem]
    reputation_minimum6: int
    reputation_direction6: Literal["source", "destination"]
    rtp_nat: Literal["disable", "enable"]
    rtp_addr: list[PolicyRtpaddrItem]
    send_deny_packet: Literal["disable", "enable"]
    firewall_session_dirty: Literal["check-all", "check-new"]
    schedule: str
    schedule_timeout: Literal["enable", "disable"]
    policy_expiry: Literal["enable", "disable"]
    policy_expiry_date: str
    policy_expiry_date_utc: str
    service: list[PolicyServiceItem]
    tos_mask: str
    tos: str
    tos_negate: Literal["enable", "disable"]
    anti_replay: Literal["enable", "disable"]
    tcp_session_without_syn: Literal["all", "data-only", "disable"]
    geoip_anycast: Literal["enable", "disable"]
    geoip_match: Literal["physical-location", "registered-location"]
    dynamic_shaping: Literal["enable", "disable"]
    passive_wan_health_measurement: Literal["enable", "disable"]
    app_monitor: Literal["enable", "disable"]
    utm_status: Literal["enable", "disable"]
    inspection_mode: Literal["proxy", "flow"]
    http_policy_redirect: Literal["enable", "disable", "legacy"]
    ssh_policy_redirect: Literal["enable", "disable"]
    ztna_policy_redirect: Literal["enable", "disable"]
    webproxy_profile: str
    profile_type: Literal["single", "group"]
    profile_group: str
    profile_protocol_options: str
    ssl_ssh_profile: str
    av_profile: str
    webfilter_profile: str
    dnsfilter_profile: str
    emailfilter_profile: str
    dlp_profile: str
    file_filter_profile: str
    ips_sensor: str
    application_list: str
    voip_profile: str
    ips_voip_filter: str
    sctp_filter_profile: str
    diameter_filter_profile: str
    virtual_patch_profile: str
    icap_profile: str
    videofilter_profile: str
    waf_profile: str
    ssh_filter_profile: str
    casb_profile: str
    logtraffic: Literal["all", "utm", "disable"]
    logtraffic_start: Literal["enable", "disable"]
    log_http_transaction: Literal["enable", "disable"]
    capture_packet: Literal["enable", "disable"]
    auto_asic_offload: Literal["enable", "disable"]
    wanopt: Literal["enable", "disable"]
    wanopt_detection: Literal["active", "passive", "off"]
    wanopt_passive_opt: Literal["default", "transparent", "non-transparent"]
    wanopt_profile: str
    wanopt_peer: str
    webcache: Literal["enable", "disable"]
    webcache_https: Literal["disable", "enable"]
    webproxy_forward_server: str
    traffic_shaper: str
    traffic_shaper_reverse: str
    per_ip_shaper: str
    nat: Literal["enable", "disable"]
    pcp_outbound: Literal["enable", "disable"]
    pcp_inbound: Literal["enable", "disable"]
    pcp_poolname: list[PolicyPcppoolnameItem]
    permit_any_host: Literal["enable", "disable"]
    permit_stun_host: Literal["enable", "disable"]
    fixedport: Literal["enable", "disable"]
    port_preserve: Literal["enable", "disable"]
    port_random: Literal["enable", "disable"]
    ippool: Literal["enable", "disable"]
    poolname: list[PolicyPoolnameItem]
    poolname6: list[PolicyPoolname6Item]
    session_ttl: str
    vlan_cos_fwd: int
    vlan_cos_rev: int
    inbound: Literal["enable", "disable"]
    outbound: Literal["enable", "disable"]
    natinbound: Literal["enable", "disable"]
    natoutbound: Literal["enable", "disable"]
    fec: Literal["enable", "disable"]
    wccp: Literal["enable", "disable"]
    ntlm: Literal["enable", "disable"]
    ntlm_guest: Literal["enable", "disable"]
    ntlm_enabled_browsers: list[PolicyNtlmenabledbrowsersItem]
    fsso_agent_for_ntlm: str
    groups: list[PolicyGroupsItem]
    users: list[PolicyUsersItem]
    fsso_groups: list[PolicyFssogroupsItem]
    auth_path: Literal["enable", "disable"]
    disclaimer: Literal["enable", "disable"]
    email_collect: Literal["enable", "disable"]
    vpntunnel: str
    natip: str
    match_vip: Literal["enable", "disable"]
    match_vip_only: Literal["enable", "disable"]
    diffserv_copy: Literal["enable", "disable"]
    diffserv_forward: Literal["enable", "disable"]
    diffserv_reverse: Literal["enable", "disable"]
    diffservcode_forward: str
    diffservcode_rev: str
    tcp_mss_sender: int
    tcp_mss_receiver: int
    comments: str
    auth_cert: str
    auth_redirect_addr: str
    redirect_url: str
    identity_based_route: str
    block_notification: Literal["enable", "disable"]
    custom_log_fields: list[PolicyCustomlogfieldsItem]
    replacemsg_override_group: str
    srcaddr_negate: Literal["enable", "disable"]
    srcaddr6_negate: Literal["enable", "disable"]
    dstaddr_negate: Literal["enable", "disable"]
    dstaddr6_negate: Literal["enable", "disable"]
    ztna_ems_tag_negate: Literal["enable", "disable"]
    service_negate: Literal["enable", "disable"]
    internet_service_negate: Literal["enable", "disable"]
    internet_service_src_negate: Literal["enable", "disable"]
    internet_service6_negate: Literal["enable", "disable"]
    internet_service6_src_negate: Literal["enable", "disable"]
    timeout_send_rst: Literal["enable", "disable"]
    captive_portal_exempt: Literal["enable", "disable"]
    decrypted_traffic_mirror: str
    dsri: Literal["enable", "disable"]
    radius_mac_auth_bypass: Literal["enable", "disable"]
    radius_ip_auth_bypass: Literal["enable", "disable"]
    delay_tcp_npu_session: Literal["enable", "disable"]
    vlan_filter: str
    sgt_check: Literal["enable", "disable"]
    sgt: list[PolicySgtItem]
    internet_service_fortiguard: list[PolicyInternetservicefortiguardItem]
    internet_service_src_fortiguard: list[PolicyInternetservicesrcfortiguardItem]
    internet_service6_fortiguard: list[PolicyInternetservice6fortiguardItem]
    internet_service6_src_fortiguard: list[PolicyInternetservice6srcfortiguardItem]


# ================================================================
# Response Types (Class for attribute access)
# ================================================================


class PolicyObject(FortiObject):
    """Typed FortiObject for Policy with field access."""
    policyid: int
    status: Literal["enable", "disable"]
    name: str
    uuid: str
    srcintf: list[PolicySrcintfItem]
    dstintf: list[PolicyDstintfItem]
    action: Literal["accept", "deny", "ipsec"]
    nat64: Literal["enable", "disable"]
    nat46: Literal["enable", "disable"]
    ztna_status: Literal["enable", "disable"]
    ztna_device_ownership: Literal["enable", "disable"]
    srcaddr: list[PolicySrcaddrItem]
    dstaddr: list[PolicyDstaddrItem]
    srcaddr6: list[PolicySrcaddr6Item]
    dstaddr6: list[PolicyDstaddr6Item]
    ztna_ems_tag: list[PolicyZtnaemstagItem]
    ztna_ems_tag_secondary: list[PolicyZtnaemstagsecondaryItem]
    ztna_tags_match_logic: Literal["or", "and"]
    ztna_geo_tag: list[PolicyZtnageotagItem]
    internet_service: Literal["enable", "disable"]
    internet_service_name: list[PolicyInternetservicenameItem]
    internet_service_group: list[PolicyInternetservicegroupItem]
    internet_service_custom: list[PolicyInternetservicecustomItem]
    network_service_dynamic: list[PolicyNetworkservicedynamicItem]
    internet_service_custom_group: list[PolicyInternetservicecustomgroupItem]
    internet_service_src: Literal["enable", "disable"]
    internet_service_src_name: list[PolicyInternetservicesrcnameItem]
    internet_service_src_group: list[PolicyInternetservicesrcgroupItem]
    internet_service_src_custom: list[PolicyInternetservicesrccustomItem]
    network_service_src_dynamic: list[PolicyNetworkservicesrcdynamicItem]
    internet_service_src_custom_group: list[PolicyInternetservicesrccustomgroupItem]
    reputation_minimum: int
    reputation_direction: Literal["source", "destination"]
    src_vendor_mac: list[PolicySrcvendormacItem]
    internet_service6: Literal["enable", "disable"]
    internet_service6_name: list[PolicyInternetservice6nameItem]
    internet_service6_group: list[PolicyInternetservice6groupItem]
    internet_service6_custom: list[PolicyInternetservice6customItem]
    internet_service6_custom_group: list[PolicyInternetservice6customgroupItem]
    internet_service6_src: Literal["enable", "disable"]
    internet_service6_src_name: list[PolicyInternetservice6srcnameItem]
    internet_service6_src_group: list[PolicyInternetservice6srcgroupItem]
    internet_service6_src_custom: list[PolicyInternetservice6srccustomItem]
    internet_service6_src_custom_group: list[PolicyInternetservice6srccustomgroupItem]
    reputation_minimum6: int
    reputation_direction6: Literal["source", "destination"]
    rtp_nat: Literal["disable", "enable"]
    rtp_addr: list[PolicyRtpaddrItem]
    send_deny_packet: Literal["disable", "enable"]
    firewall_session_dirty: Literal["check-all", "check-new"]
    schedule: str
    schedule_timeout: Literal["enable", "disable"]
    policy_expiry: Literal["enable", "disable"]
    policy_expiry_date: str
    policy_expiry_date_utc: str
    service: list[PolicyServiceItem]
    tos_mask: str
    tos: str
    tos_negate: Literal["enable", "disable"]
    anti_replay: Literal["enable", "disable"]
    tcp_session_without_syn: Literal["all", "data-only", "disable"]
    geoip_anycast: Literal["enable", "disable"]
    geoip_match: Literal["physical-location", "registered-location"]
    dynamic_shaping: Literal["enable", "disable"]
    passive_wan_health_measurement: Literal["enable", "disable"]
    app_monitor: Literal["enable", "disable"]
    utm_status: Literal["enable", "disable"]
    inspection_mode: Literal["proxy", "flow"]
    http_policy_redirect: Literal["enable", "disable", "legacy"]
    ssh_policy_redirect: Literal["enable", "disable"]
    ztna_policy_redirect: Literal["enable", "disable"]
    webproxy_profile: str
    profile_type: Literal["single", "group"]
    profile_group: str
    profile_protocol_options: str
    ssl_ssh_profile: str
    av_profile: str
    webfilter_profile: str
    dnsfilter_profile: str
    emailfilter_profile: str
    dlp_profile: str
    file_filter_profile: str
    ips_sensor: str
    application_list: str
    voip_profile: str
    ips_voip_filter: str
    sctp_filter_profile: str
    diameter_filter_profile: str
    virtual_patch_profile: str
    icap_profile: str
    videofilter_profile: str
    waf_profile: str
    ssh_filter_profile: str
    casb_profile: str
    logtraffic: Literal["all", "utm", "disable"]
    logtraffic_start: Literal["enable", "disable"]
    log_http_transaction: Literal["enable", "disable"]
    capture_packet: Literal["enable", "disable"]
    auto_asic_offload: Literal["enable", "disable"]
    wanopt: Literal["enable", "disable"]
    wanopt_detection: Literal["active", "passive", "off"]
    wanopt_passive_opt: Literal["default", "transparent", "non-transparent"]
    wanopt_profile: str
    wanopt_peer: str
    webcache: Literal["enable", "disable"]
    webcache_https: Literal["disable", "enable"]
    webproxy_forward_server: str
    traffic_shaper: str
    traffic_shaper_reverse: str
    per_ip_shaper: str
    nat: Literal["enable", "disable"]
    pcp_outbound: Literal["enable", "disable"]
    pcp_inbound: Literal["enable", "disable"]
    pcp_poolname: list[PolicyPcppoolnameItem]
    permit_any_host: Literal["enable", "disable"]
    permit_stun_host: Literal["enable", "disable"]
    fixedport: Literal["enable", "disable"]
    port_preserve: Literal["enable", "disable"]
    port_random: Literal["enable", "disable"]
    ippool: Literal["enable", "disable"]
    poolname: list[PolicyPoolnameItem]
    poolname6: list[PolicyPoolname6Item]
    session_ttl: str
    vlan_cos_fwd: int
    vlan_cos_rev: int
    inbound: Literal["enable", "disable"]
    outbound: Literal["enable", "disable"]
    natinbound: Literal["enable", "disable"]
    natoutbound: Literal["enable", "disable"]
    fec: Literal["enable", "disable"]
    wccp: Literal["enable", "disable"]
    ntlm: Literal["enable", "disable"]
    ntlm_guest: Literal["enable", "disable"]
    ntlm_enabled_browsers: list[PolicyNtlmenabledbrowsersItem]
    fsso_agent_for_ntlm: str
    groups: list[PolicyGroupsItem]
    users: list[PolicyUsersItem]
    fsso_groups: list[PolicyFssogroupsItem]
    auth_path: Literal["enable", "disable"]
    disclaimer: Literal["enable", "disable"]
    email_collect: Literal["enable", "disable"]
    vpntunnel: str
    natip: str
    match_vip: Literal["enable", "disable"]
    match_vip_only: Literal["enable", "disable"]
    diffserv_copy: Literal["enable", "disable"]
    diffserv_forward: Literal["enable", "disable"]
    diffserv_reverse: Literal["enable", "disable"]
    diffservcode_forward: str
    diffservcode_rev: str
    tcp_mss_sender: int
    tcp_mss_receiver: int
    comments: str
    auth_cert: str
    auth_redirect_addr: str
    redirect_url: str
    identity_based_route: str
    block_notification: Literal["enable", "disable"]
    custom_log_fields: list[PolicyCustomlogfieldsItem]
    replacemsg_override_group: str
    srcaddr_negate: Literal["enable", "disable"]
    srcaddr6_negate: Literal["enable", "disable"]
    dstaddr_negate: Literal["enable", "disable"]
    dstaddr6_negate: Literal["enable", "disable"]
    ztna_ems_tag_negate: Literal["enable", "disable"]
    service_negate: Literal["enable", "disable"]
    internet_service_negate: Literal["enable", "disable"]
    internet_service_src_negate: Literal["enable", "disable"]
    internet_service6_negate: Literal["enable", "disable"]
    internet_service6_src_negate: Literal["enable", "disable"]
    timeout_send_rst: Literal["enable", "disable"]
    captive_portal_exempt: Literal["enable", "disable"]
    decrypted_traffic_mirror: str
    dsri: Literal["enable", "disable"]
    radius_mac_auth_bypass: Literal["enable", "disable"]
    radius_ip_auth_bypass: Literal["enable", "disable"]
    delay_tcp_npu_session: Literal["enable", "disable"]
    vlan_filter: str
    sgt_check: Literal["enable", "disable"]
    sgt: list[PolicySgtItem]
    internet_service_fortiguard: list[PolicyInternetservicefortiguardItem]
    internet_service_src_fortiguard: list[PolicyInternetservicesrcfortiguardItem]
    internet_service6_fortiguard: list[PolicyInternetservice6fortiguardItem]
    internet_service6_src_fortiguard: list[PolicyInternetservice6srcfortiguardItem]


# ================================================================
# Main Endpoint Class
# ================================================================

class Policy:
    """
    
    Endpoint: firewall/policy
    Category: cmdb
    MKey: policyid
    """
    
    # Class attributes for introspection
    endpoint: ClassVar[str] = ...
    path: ClassVar[str] = ...
    category: ClassVar[str] = ...
    mkey: ClassVar[str] = ...
    capabilities: ClassVar[dict[str, Any]] = ...
    
    def __init__(self, client: Any) -> None:
        """Initialize endpoint with HTTP client."""
        ...
    
    # ================================================================
    # GET Methods
    # ================================================================
    
    # CMDB with mkey - overloads for single vs list returns
    @overload
    def get(
        self,
        policyid: int,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> PolicyObject: ...
    
    @overload
    def get(
        self,
        *,
        filter: str | list[str] | None = ...,
        count: int | None = ...,
        start: int | None = ...,
        payload_dict: dict[str, Any] | None = ...,
        range: list[int] | None = ...,
        sort: str | None = ...,
        format: str | None = ...,
        action: str | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObjectList[PolicyObject]: ...
    
    def get_schema(
        self,
        vdom: str | None = ...,
        format: str = ...,
    ) -> FortiObject: ...

    # ================================================================
    # POST Method
    # ================================================================
    
    def post(
        self,
        payload_dict: PolicyPayload | None = ...,
        policyid: int | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        name: str | None = ...,
        uuid: str | None = ...,
        srcintf: str | list[str] | list[dict[str, Any]] | list[PolicySrcintfItem] | None = ...,
        dstintf: str | list[str] | list[dict[str, Any]] | list[PolicyDstintfItem] | None = ...,
        action: Literal["accept", "deny", "ipsec"] | None = ...,
        nat64: Literal["enable", "disable"] | None = ...,
        nat46: Literal["enable", "disable"] | None = ...,
        ztna_status: Literal["enable", "disable"] | None = ...,
        ztna_device_ownership: Literal["enable", "disable"] | None = ...,
        srcaddr: str | list[str] | list[dict[str, Any]] | list[PolicySrcaddrItem] | None = ...,
        dstaddr: str | list[str] | list[dict[str, Any]] | list[PolicyDstaddrItem] | None = ...,
        srcaddr6: str | list[str] | list[dict[str, Any]] | list[PolicySrcaddr6Item] | None = ...,
        dstaddr6: str | list[str] | list[dict[str, Any]] | list[PolicyDstaddr6Item] | None = ...,
        ztna_ems_tag: str | list[str] | list[dict[str, Any]] | list[PolicyZtnaemstagItem] | None = ...,
        ztna_ems_tag_secondary: str | list[str] | list[dict[str, Any]] | list[PolicyZtnaemstagsecondaryItem] | None = ...,
        ztna_tags_match_logic: Literal["or", "and"] | None = ...,
        ztna_geo_tag: str | list[str] | list[dict[str, Any]] | list[PolicyZtnageotagItem] | None = ...,
        internet_service: Literal["enable", "disable"] | None = ...,
        internet_service_name: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicenameItem] | None = ...,
        internet_service_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicegroupItem] | None = ...,
        internet_service_custom: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicecustomItem] | None = ...,
        network_service_dynamic: str | list[str] | list[dict[str, Any]] | list[PolicyNetworkservicedynamicItem] | None = ...,
        internet_service_custom_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicecustomgroupItem] | None = ...,
        internet_service_src: Literal["enable", "disable"] | None = ...,
        internet_service_src_name: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicesrcnameItem] | None = ...,
        internet_service_src_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicesrcgroupItem] | None = ...,
        internet_service_src_custom: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicesrccustomItem] | None = ...,
        network_service_src_dynamic: str | list[str] | list[dict[str, Any]] | list[PolicyNetworkservicesrcdynamicItem] | None = ...,
        internet_service_src_custom_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicesrccustomgroupItem] | None = ...,
        reputation_minimum: int | None = ...,
        reputation_direction: Literal["source", "destination"] | None = ...,
        src_vendor_mac: str | list[str] | list[dict[str, Any]] | list[PolicySrcvendormacItem] | None = ...,
        internet_service6: Literal["enable", "disable"] | None = ...,
        internet_service6_name: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6nameItem] | None = ...,
        internet_service6_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6groupItem] | None = ...,
        internet_service6_custom: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6customItem] | None = ...,
        internet_service6_custom_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6customgroupItem] | None = ...,
        internet_service6_src: Literal["enable", "disable"] | None = ...,
        internet_service6_src_name: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6srcnameItem] | None = ...,
        internet_service6_src_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6srcgroupItem] | None = ...,
        internet_service6_src_custom: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6srccustomItem] | None = ...,
        internet_service6_src_custom_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6srccustomgroupItem] | None = ...,
        reputation_minimum6: int | None = ...,
        reputation_direction6: Literal["source", "destination"] | None = ...,
        rtp_nat: Literal["disable", "enable"] | None = ...,
        rtp_addr: str | list[str] | list[dict[str, Any]] | list[PolicyRtpaddrItem] | None = ...,
        send_deny_packet: Literal["disable", "enable"] | None = ...,
        firewall_session_dirty: Literal["check-all", "check-new"] | None = ...,
        schedule: str | None = ...,
        schedule_timeout: Literal["enable", "disable"] | None = ...,
        policy_expiry: Literal["enable", "disable"] | None = ...,
        policy_expiry_date: str | None = ...,
        policy_expiry_date_utc: str | None = ...,
        service: str | list[str] | list[dict[str, Any]] | list[PolicyServiceItem] | None = ...,
        tos_mask: str | None = ...,
        tos: str | None = ...,
        tos_negate: Literal["enable", "disable"] | None = ...,
        anti_replay: Literal["enable", "disable"] | None = ...,
        tcp_session_without_syn: Literal["all", "data-only", "disable"] | None = ...,
        geoip_anycast: Literal["enable", "disable"] | None = ...,
        geoip_match: Literal["physical-location", "registered-location"] | None = ...,
        dynamic_shaping: Literal["enable", "disable"] | None = ...,
        passive_wan_health_measurement: Literal["enable", "disable"] | None = ...,
        app_monitor: Literal["enable", "disable"] | None = ...,
        utm_status: Literal["enable", "disable"] | None = ...,
        inspection_mode: Literal["proxy", "flow"] | None = ...,
        http_policy_redirect: Literal["enable", "disable", "legacy"] | None = ...,
        ssh_policy_redirect: Literal["enable", "disable"] | None = ...,
        ztna_policy_redirect: Literal["enable", "disable"] | None = ...,
        webproxy_profile: str | None = ...,
        profile_type: Literal["single", "group"] | None = ...,
        profile_group: str | None = ...,
        profile_protocol_options: str | None = ...,
        ssl_ssh_profile: str | None = ...,
        av_profile: str | None = ...,
        webfilter_profile: str | None = ...,
        dnsfilter_profile: str | None = ...,
        emailfilter_profile: str | None = ...,
        dlp_profile: str | None = ...,
        file_filter_profile: str | None = ...,
        ips_sensor: str | None = ...,
        application_list: str | None = ...,
        voip_profile: str | None = ...,
        ips_voip_filter: str | None = ...,
        sctp_filter_profile: str | None = ...,
        diameter_filter_profile: str | None = ...,
        virtual_patch_profile: str | None = ...,
        icap_profile: str | None = ...,
        videofilter_profile: str | None = ...,
        waf_profile: str | None = ...,
        ssh_filter_profile: str | None = ...,
        casb_profile: str | None = ...,
        logtraffic: Literal["all", "utm", "disable"] | None = ...,
        logtraffic_start: Literal["enable", "disable"] | None = ...,
        log_http_transaction: Literal["enable", "disable"] | None = ...,
        capture_packet: Literal["enable", "disable"] | None = ...,
        auto_asic_offload: Literal["enable", "disable"] | None = ...,
        wanopt: Literal["enable", "disable"] | None = ...,
        wanopt_detection: Literal["active", "passive", "off"] | None = ...,
        wanopt_passive_opt: Literal["default", "transparent", "non-transparent"] | None = ...,
        wanopt_profile: str | None = ...,
        wanopt_peer: str | None = ...,
        webcache: Literal["enable", "disable"] | None = ...,
        webcache_https: Literal["disable", "enable"] | None = ...,
        webproxy_forward_server: str | None = ...,
        traffic_shaper: str | None = ...,
        traffic_shaper_reverse: str | None = ...,
        per_ip_shaper: str | None = ...,
        nat: Literal["enable", "disable"] | None = ...,
        pcp_outbound: Literal["enable", "disable"] | None = ...,
        pcp_inbound: Literal["enable", "disable"] | None = ...,
        pcp_poolname: str | list[str] | list[dict[str, Any]] | list[PolicyPcppoolnameItem] | None = ...,
        permit_any_host: Literal["enable", "disable"] | None = ...,
        permit_stun_host: Literal["enable", "disable"] | None = ...,
        fixedport: Literal["enable", "disable"] | None = ...,
        port_preserve: Literal["enable", "disable"] | None = ...,
        port_random: Literal["enable", "disable"] | None = ...,
        ippool: Literal["enable", "disable"] | None = ...,
        poolname: str | list[str] | list[dict[str, Any]] | list[PolicyPoolnameItem] | None = ...,
        poolname6: str | list[str] | list[dict[str, Any]] | list[PolicyPoolname6Item] | None = ...,
        session_ttl: str | None = ...,
        vlan_cos_fwd: int | None = ...,
        vlan_cos_rev: int | None = ...,
        inbound: Literal["enable", "disable"] | None = ...,
        outbound: Literal["enable", "disable"] | None = ...,
        natinbound: Literal["enable", "disable"] | None = ...,
        natoutbound: Literal["enable", "disable"] | None = ...,
        fec: Literal["enable", "disable"] | None = ...,
        wccp: Literal["enable", "disable"] | None = ...,
        ntlm: Literal["enable", "disable"] | None = ...,
        ntlm_guest: Literal["enable", "disable"] | None = ...,
        ntlm_enabled_browsers: str | list[str] | list[dict[str, Any]] | list[PolicyNtlmenabledbrowsersItem] | None = ...,
        fsso_agent_for_ntlm: str | None = ...,
        groups: str | list[str] | list[dict[str, Any]] | list[PolicyGroupsItem] | None = ...,
        users: str | list[str] | list[dict[str, Any]] | list[PolicyUsersItem] | None = ...,
        fsso_groups: str | list[str] | list[dict[str, Any]] | list[PolicyFssogroupsItem] | None = ...,
        auth_path: Literal["enable", "disable"] | None = ...,
        disclaimer: Literal["enable", "disable"] | None = ...,
        email_collect: Literal["enable", "disable"] | None = ...,
        vpntunnel: str | None = ...,
        natip: str | None = ...,
        match_vip: Literal["enable", "disable"] | None = ...,
        match_vip_only: Literal["enable", "disable"] | None = ...,
        diffserv_copy: Literal["enable", "disable"] | None = ...,
        diffserv_forward: Literal["enable", "disable"] | None = ...,
        diffserv_reverse: Literal["enable", "disable"] | None = ...,
        diffservcode_forward: str | None = ...,
        diffservcode_rev: str | None = ...,
        tcp_mss_sender: int | None = ...,
        tcp_mss_receiver: int | None = ...,
        comments: str | None = ...,
        auth_cert: str | None = ...,
        auth_redirect_addr: str | None = ...,
        redirect_url: str | None = ...,
        identity_based_route: str | None = ...,
        block_notification: Literal["enable", "disable"] | None = ...,
        custom_log_fields: str | list[str] | list[dict[str, Any]] | list[PolicyCustomlogfieldsItem] | None = ...,
        replacemsg_override_group: str | None = ...,
        srcaddr_negate: Literal["enable", "disable"] | None = ...,
        srcaddr6_negate: Literal["enable", "disable"] | None = ...,
        dstaddr_negate: Literal["enable", "disable"] | None = ...,
        dstaddr6_negate: Literal["enable", "disable"] | None = ...,
        ztna_ems_tag_negate: Literal["enable", "disable"] | None = ...,
        service_negate: Literal["enable", "disable"] | None = ...,
        internet_service_negate: Literal["enable", "disable"] | None = ...,
        internet_service_src_negate: Literal["enable", "disable"] | None = ...,
        internet_service6_negate: Literal["enable", "disable"] | None = ...,
        internet_service6_src_negate: Literal["enable", "disable"] | None = ...,
        timeout_send_rst: Literal["enable", "disable"] | None = ...,
        captive_portal_exempt: Literal["enable", "disable"] | None = ...,
        decrypted_traffic_mirror: str | None = ...,
        dsri: Literal["enable", "disable"] | None = ...,
        radius_mac_auth_bypass: Literal["enable", "disable"] | None = ...,
        radius_ip_auth_bypass: Literal["enable", "disable"] | None = ...,
        delay_tcp_npu_session: Literal["enable", "disable"] | None = ...,
        vlan_filter: str | None = ...,
        sgt_check: Literal["enable", "disable"] | None = ...,
        sgt: str | list[str] | list[dict[str, Any]] | list[PolicySgtItem] | None = ...,
        internet_service_fortiguard: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicefortiguardItem] | None = ...,
        internet_service_src_fortiguard: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicesrcfortiguardItem] | None = ...,
        internet_service6_fortiguard: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6fortiguardItem] | None = ...,
        internet_service6_src_fortiguard: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6srcfortiguardItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> PolicyObject: ...

    # ================================================================
    # PUT Method
    # ================================================================
    
    def put(
        self,
        payload_dict: PolicyPayload | None = ...,
        policyid: int | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        name: str | None = ...,
        uuid: str | None = ...,
        srcintf: str | list[str] | list[dict[str, Any]] | list[PolicySrcintfItem] | None = ...,
        dstintf: str | list[str] | list[dict[str, Any]] | list[PolicyDstintfItem] | None = ...,
        action: Literal["accept", "deny", "ipsec"] | None = ...,
        nat64: Literal["enable", "disable"] | None = ...,
        nat46: Literal["enable", "disable"] | None = ...,
        ztna_status: Literal["enable", "disable"] | None = ...,
        ztna_device_ownership: Literal["enable", "disable"] | None = ...,
        srcaddr: str | list[str] | list[dict[str, Any]] | list[PolicySrcaddrItem] | None = ...,
        dstaddr: str | list[str] | list[dict[str, Any]] | list[PolicyDstaddrItem] | None = ...,
        srcaddr6: str | list[str] | list[dict[str, Any]] | list[PolicySrcaddr6Item] | None = ...,
        dstaddr6: str | list[str] | list[dict[str, Any]] | list[PolicyDstaddr6Item] | None = ...,
        ztna_ems_tag: str | list[str] | list[dict[str, Any]] | list[PolicyZtnaemstagItem] | None = ...,
        ztna_ems_tag_secondary: str | list[str] | list[dict[str, Any]] | list[PolicyZtnaemstagsecondaryItem] | None = ...,
        ztna_tags_match_logic: Literal["or", "and"] | None = ...,
        ztna_geo_tag: str | list[str] | list[dict[str, Any]] | list[PolicyZtnageotagItem] | None = ...,
        internet_service: Literal["enable", "disable"] | None = ...,
        internet_service_name: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicenameItem] | None = ...,
        internet_service_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicegroupItem] | None = ...,
        internet_service_custom: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicecustomItem] | None = ...,
        network_service_dynamic: str | list[str] | list[dict[str, Any]] | list[PolicyNetworkservicedynamicItem] | None = ...,
        internet_service_custom_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicecustomgroupItem] | None = ...,
        internet_service_src: Literal["enable", "disable"] | None = ...,
        internet_service_src_name: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicesrcnameItem] | None = ...,
        internet_service_src_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicesrcgroupItem] | None = ...,
        internet_service_src_custom: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicesrccustomItem] | None = ...,
        network_service_src_dynamic: str | list[str] | list[dict[str, Any]] | list[PolicyNetworkservicesrcdynamicItem] | None = ...,
        internet_service_src_custom_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicesrccustomgroupItem] | None = ...,
        reputation_minimum: int | None = ...,
        reputation_direction: Literal["source", "destination"] | None = ...,
        src_vendor_mac: str | list[str] | list[dict[str, Any]] | list[PolicySrcvendormacItem] | None = ...,
        internet_service6: Literal["enable", "disable"] | None = ...,
        internet_service6_name: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6nameItem] | None = ...,
        internet_service6_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6groupItem] | None = ...,
        internet_service6_custom: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6customItem] | None = ...,
        internet_service6_custom_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6customgroupItem] | None = ...,
        internet_service6_src: Literal["enable", "disable"] | None = ...,
        internet_service6_src_name: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6srcnameItem] | None = ...,
        internet_service6_src_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6srcgroupItem] | None = ...,
        internet_service6_src_custom: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6srccustomItem] | None = ...,
        internet_service6_src_custom_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6srccustomgroupItem] | None = ...,
        reputation_minimum6: int | None = ...,
        reputation_direction6: Literal["source", "destination"] | None = ...,
        rtp_nat: Literal["disable", "enable"] | None = ...,
        rtp_addr: str | list[str] | list[dict[str, Any]] | list[PolicyRtpaddrItem] | None = ...,
        send_deny_packet: Literal["disable", "enable"] | None = ...,
        firewall_session_dirty: Literal["check-all", "check-new"] | None = ...,
        schedule: str | None = ...,
        schedule_timeout: Literal["enable", "disable"] | None = ...,
        policy_expiry: Literal["enable", "disable"] | None = ...,
        policy_expiry_date: str | None = ...,
        policy_expiry_date_utc: str | None = ...,
        service: str | list[str] | list[dict[str, Any]] | list[PolicyServiceItem] | None = ...,
        tos_mask: str | None = ...,
        tos: str | None = ...,
        tos_negate: Literal["enable", "disable"] | None = ...,
        anti_replay: Literal["enable", "disable"] | None = ...,
        tcp_session_without_syn: Literal["all", "data-only", "disable"] | None = ...,
        geoip_anycast: Literal["enable", "disable"] | None = ...,
        geoip_match: Literal["physical-location", "registered-location"] | None = ...,
        dynamic_shaping: Literal["enable", "disable"] | None = ...,
        passive_wan_health_measurement: Literal["enable", "disable"] | None = ...,
        app_monitor: Literal["enable", "disable"] | None = ...,
        utm_status: Literal["enable", "disable"] | None = ...,
        inspection_mode: Literal["proxy", "flow"] | None = ...,
        http_policy_redirect: Literal["enable", "disable", "legacy"] | None = ...,
        ssh_policy_redirect: Literal["enable", "disable"] | None = ...,
        ztna_policy_redirect: Literal["enable", "disable"] | None = ...,
        webproxy_profile: str | None = ...,
        profile_type: Literal["single", "group"] | None = ...,
        profile_group: str | None = ...,
        profile_protocol_options: str | None = ...,
        ssl_ssh_profile: str | None = ...,
        av_profile: str | None = ...,
        webfilter_profile: str | None = ...,
        dnsfilter_profile: str | None = ...,
        emailfilter_profile: str | None = ...,
        dlp_profile: str | None = ...,
        file_filter_profile: str | None = ...,
        ips_sensor: str | None = ...,
        application_list: str | None = ...,
        voip_profile: str | None = ...,
        ips_voip_filter: str | None = ...,
        sctp_filter_profile: str | None = ...,
        diameter_filter_profile: str | None = ...,
        virtual_patch_profile: str | None = ...,
        icap_profile: str | None = ...,
        videofilter_profile: str | None = ...,
        waf_profile: str | None = ...,
        ssh_filter_profile: str | None = ...,
        casb_profile: str | None = ...,
        logtraffic: Literal["all", "utm", "disable"] | None = ...,
        logtraffic_start: Literal["enable", "disable"] | None = ...,
        log_http_transaction: Literal["enable", "disable"] | None = ...,
        capture_packet: Literal["enable", "disable"] | None = ...,
        auto_asic_offload: Literal["enable", "disable"] | None = ...,
        wanopt: Literal["enable", "disable"] | None = ...,
        wanopt_detection: Literal["active", "passive", "off"] | None = ...,
        wanopt_passive_opt: Literal["default", "transparent", "non-transparent"] | None = ...,
        wanopt_profile: str | None = ...,
        wanopt_peer: str | None = ...,
        webcache: Literal["enable", "disable"] | None = ...,
        webcache_https: Literal["disable", "enable"] | None = ...,
        webproxy_forward_server: str | None = ...,
        traffic_shaper: str | None = ...,
        traffic_shaper_reverse: str | None = ...,
        per_ip_shaper: str | None = ...,
        nat: Literal["enable", "disable"] | None = ...,
        pcp_outbound: Literal["enable", "disable"] | None = ...,
        pcp_inbound: Literal["enable", "disable"] | None = ...,
        pcp_poolname: str | list[str] | list[dict[str, Any]] | list[PolicyPcppoolnameItem] | None = ...,
        permit_any_host: Literal["enable", "disable"] | None = ...,
        permit_stun_host: Literal["enable", "disable"] | None = ...,
        fixedport: Literal["enable", "disable"] | None = ...,
        port_preserve: Literal["enable", "disable"] | None = ...,
        port_random: Literal["enable", "disable"] | None = ...,
        ippool: Literal["enable", "disable"] | None = ...,
        poolname: str | list[str] | list[dict[str, Any]] | list[PolicyPoolnameItem] | None = ...,
        poolname6: str | list[str] | list[dict[str, Any]] | list[PolicyPoolname6Item] | None = ...,
        session_ttl: str | None = ...,
        vlan_cos_fwd: int | None = ...,
        vlan_cos_rev: int | None = ...,
        inbound: Literal["enable", "disable"] | None = ...,
        outbound: Literal["enable", "disable"] | None = ...,
        natinbound: Literal["enable", "disable"] | None = ...,
        natoutbound: Literal["enable", "disable"] | None = ...,
        fec: Literal["enable", "disable"] | None = ...,
        wccp: Literal["enable", "disable"] | None = ...,
        ntlm: Literal["enable", "disable"] | None = ...,
        ntlm_guest: Literal["enable", "disable"] | None = ...,
        ntlm_enabled_browsers: str | list[str] | list[dict[str, Any]] | list[PolicyNtlmenabledbrowsersItem] | None = ...,
        fsso_agent_for_ntlm: str | None = ...,
        groups: str | list[str] | list[dict[str, Any]] | list[PolicyGroupsItem] | None = ...,
        users: str | list[str] | list[dict[str, Any]] | list[PolicyUsersItem] | None = ...,
        fsso_groups: str | list[str] | list[dict[str, Any]] | list[PolicyFssogroupsItem] | None = ...,
        auth_path: Literal["enable", "disable"] | None = ...,
        disclaimer: Literal["enable", "disable"] | None = ...,
        email_collect: Literal["enable", "disable"] | None = ...,
        vpntunnel: str | None = ...,
        natip: str | None = ...,
        match_vip: Literal["enable", "disable"] | None = ...,
        match_vip_only: Literal["enable", "disable"] | None = ...,
        diffserv_copy: Literal["enable", "disable"] | None = ...,
        diffserv_forward: Literal["enable", "disable"] | None = ...,
        diffserv_reverse: Literal["enable", "disable"] | None = ...,
        diffservcode_forward: str | None = ...,
        diffservcode_rev: str | None = ...,
        tcp_mss_sender: int | None = ...,
        tcp_mss_receiver: int | None = ...,
        comments: str | None = ...,
        auth_cert: str | None = ...,
        auth_redirect_addr: str | None = ...,
        redirect_url: str | None = ...,
        identity_based_route: str | None = ...,
        block_notification: Literal["enable", "disable"] | None = ...,
        custom_log_fields: str | list[str] | list[dict[str, Any]] | list[PolicyCustomlogfieldsItem] | None = ...,
        replacemsg_override_group: str | None = ...,
        srcaddr_negate: Literal["enable", "disable"] | None = ...,
        srcaddr6_negate: Literal["enable", "disable"] | None = ...,
        dstaddr_negate: Literal["enable", "disable"] | None = ...,
        dstaddr6_negate: Literal["enable", "disable"] | None = ...,
        ztna_ems_tag_negate: Literal["enable", "disable"] | None = ...,
        service_negate: Literal["enable", "disable"] | None = ...,
        internet_service_negate: Literal["enable", "disable"] | None = ...,
        internet_service_src_negate: Literal["enable", "disable"] | None = ...,
        internet_service6_negate: Literal["enable", "disable"] | None = ...,
        internet_service6_src_negate: Literal["enable", "disable"] | None = ...,
        timeout_send_rst: Literal["enable", "disable"] | None = ...,
        captive_portal_exempt: Literal["enable", "disable"] | None = ...,
        decrypted_traffic_mirror: str | None = ...,
        dsri: Literal["enable", "disable"] | None = ...,
        radius_mac_auth_bypass: Literal["enable", "disable"] | None = ...,
        radius_ip_auth_bypass: Literal["enable", "disable"] | None = ...,
        delay_tcp_npu_session: Literal["enable", "disable"] | None = ...,
        vlan_filter: str | None = ...,
        sgt_check: Literal["enable", "disable"] | None = ...,
        sgt: str | list[str] | list[dict[str, Any]] | list[PolicySgtItem] | None = ...,
        internet_service_fortiguard: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicefortiguardItem] | None = ...,
        internet_service_src_fortiguard: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicesrcfortiguardItem] | None = ...,
        internet_service6_fortiguard: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6fortiguardItem] | None = ...,
        internet_service6_src_fortiguard: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6srcfortiguardItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> PolicyObject: ...

    # ================================================================
    # DELETE Method
    # ================================================================
    
    def delete(
        self,
        policyid: int | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject: ...

    # ================================================================
    # Utility Methods
    # ================================================================
    
    def exists(
        self,
        policyid: int,
        vdom: str | bool | None = ...,
    ) -> bool: ...
    
    def set(
        self,
        payload_dict: PolicyPayload | None = ...,
        policyid: int | None = ...,
        status: Literal["enable", "disable"] | None = ...,
        name: str | None = ...,
        uuid: str | None = ...,
        srcintf: str | list[str] | list[dict[str, Any]] | list[PolicySrcintfItem] | None = ...,
        dstintf: str | list[str] | list[dict[str, Any]] | list[PolicyDstintfItem] | None = ...,
        action: Literal["accept", "deny", "ipsec"] | None = ...,
        nat64: Literal["enable", "disable"] | None = ...,
        nat46: Literal["enable", "disable"] | None = ...,
        ztna_status: Literal["enable", "disable"] | None = ...,
        ztna_device_ownership: Literal["enable", "disable"] | None = ...,
        srcaddr: str | list[str] | list[dict[str, Any]] | list[PolicySrcaddrItem] | None = ...,
        dstaddr: str | list[str] | list[dict[str, Any]] | list[PolicyDstaddrItem] | None = ...,
        srcaddr6: str | list[str] | list[dict[str, Any]] | list[PolicySrcaddr6Item] | None = ...,
        dstaddr6: str | list[str] | list[dict[str, Any]] | list[PolicyDstaddr6Item] | None = ...,
        ztna_ems_tag: str | list[str] | list[dict[str, Any]] | list[PolicyZtnaemstagItem] | None = ...,
        ztna_ems_tag_secondary: str | list[str] | list[dict[str, Any]] | list[PolicyZtnaemstagsecondaryItem] | None = ...,
        ztna_tags_match_logic: Literal["or", "and"] | None = ...,
        ztna_geo_tag: str | list[str] | list[dict[str, Any]] | list[PolicyZtnageotagItem] | None = ...,
        internet_service: Literal["enable", "disable"] | None = ...,
        internet_service_name: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicenameItem] | None = ...,
        internet_service_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicegroupItem] | None = ...,
        internet_service_custom: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicecustomItem] | None = ...,
        network_service_dynamic: str | list[str] | list[dict[str, Any]] | list[PolicyNetworkservicedynamicItem] | None = ...,
        internet_service_custom_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicecustomgroupItem] | None = ...,
        internet_service_src: Literal["enable", "disable"] | None = ...,
        internet_service_src_name: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicesrcnameItem] | None = ...,
        internet_service_src_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicesrcgroupItem] | None = ...,
        internet_service_src_custom: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicesrccustomItem] | None = ...,
        network_service_src_dynamic: str | list[str] | list[dict[str, Any]] | list[PolicyNetworkservicesrcdynamicItem] | None = ...,
        internet_service_src_custom_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicesrccustomgroupItem] | None = ...,
        reputation_minimum: int | None = ...,
        reputation_direction: Literal["source", "destination"] | None = ...,
        src_vendor_mac: str | list[str] | list[dict[str, Any]] | list[PolicySrcvendormacItem] | None = ...,
        internet_service6: Literal["enable", "disable"] | None = ...,
        internet_service6_name: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6nameItem] | None = ...,
        internet_service6_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6groupItem] | None = ...,
        internet_service6_custom: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6customItem] | None = ...,
        internet_service6_custom_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6customgroupItem] | None = ...,
        internet_service6_src: Literal["enable", "disable"] | None = ...,
        internet_service6_src_name: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6srcnameItem] | None = ...,
        internet_service6_src_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6srcgroupItem] | None = ...,
        internet_service6_src_custom: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6srccustomItem] | None = ...,
        internet_service6_src_custom_group: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6srccustomgroupItem] | None = ...,
        reputation_minimum6: int | None = ...,
        reputation_direction6: Literal["source", "destination"] | None = ...,
        rtp_nat: Literal["disable", "enable"] | None = ...,
        rtp_addr: str | list[str] | list[dict[str, Any]] | list[PolicyRtpaddrItem] | None = ...,
        send_deny_packet: Literal["disable", "enable"] | None = ...,
        firewall_session_dirty: Literal["check-all", "check-new"] | None = ...,
        schedule: str | None = ...,
        schedule_timeout: Literal["enable", "disable"] | None = ...,
        policy_expiry: Literal["enable", "disable"] | None = ...,
        policy_expiry_date: str | None = ...,
        policy_expiry_date_utc: str | None = ...,
        service: str | list[str] | list[dict[str, Any]] | list[PolicyServiceItem] | None = ...,
        tos_mask: str | None = ...,
        tos: str | None = ...,
        tos_negate: Literal["enable", "disable"] | None = ...,
        anti_replay: Literal["enable", "disable"] | None = ...,
        tcp_session_without_syn: Literal["all", "data-only", "disable"] | None = ...,
        geoip_anycast: Literal["enable", "disable"] | None = ...,
        geoip_match: Literal["physical-location", "registered-location"] | None = ...,
        dynamic_shaping: Literal["enable", "disable"] | None = ...,
        passive_wan_health_measurement: Literal["enable", "disable"] | None = ...,
        app_monitor: Literal["enable", "disable"] | None = ...,
        utm_status: Literal["enable", "disable"] | None = ...,
        inspection_mode: Literal["proxy", "flow"] | None = ...,
        http_policy_redirect: Literal["enable", "disable", "legacy"] | None = ...,
        ssh_policy_redirect: Literal["enable", "disable"] | None = ...,
        ztna_policy_redirect: Literal["enable", "disable"] | None = ...,
        webproxy_profile: str | None = ...,
        profile_type: Literal["single", "group"] | None = ...,
        profile_group: str | None = ...,
        profile_protocol_options: str | None = ...,
        ssl_ssh_profile: str | None = ...,
        av_profile: str | None = ...,
        webfilter_profile: str | None = ...,
        dnsfilter_profile: str | None = ...,
        emailfilter_profile: str | None = ...,
        dlp_profile: str | None = ...,
        file_filter_profile: str | None = ...,
        ips_sensor: str | None = ...,
        application_list: str | None = ...,
        voip_profile: str | None = ...,
        ips_voip_filter: str | None = ...,
        sctp_filter_profile: str | None = ...,
        diameter_filter_profile: str | None = ...,
        virtual_patch_profile: str | None = ...,
        icap_profile: str | None = ...,
        videofilter_profile: str | None = ...,
        waf_profile: str | None = ...,
        ssh_filter_profile: str | None = ...,
        casb_profile: str | None = ...,
        logtraffic: Literal["all", "utm", "disable"] | None = ...,
        logtraffic_start: Literal["enable", "disable"] | None = ...,
        log_http_transaction: Literal["enable", "disable"] | None = ...,
        capture_packet: Literal["enable", "disable"] | None = ...,
        auto_asic_offload: Literal["enable", "disable"] | None = ...,
        wanopt: Literal["enable", "disable"] | None = ...,
        wanopt_detection: Literal["active", "passive", "off"] | None = ...,
        wanopt_passive_opt: Literal["default", "transparent", "non-transparent"] | None = ...,
        wanopt_profile: str | None = ...,
        wanopt_peer: str | None = ...,
        webcache: Literal["enable", "disable"] | None = ...,
        webcache_https: Literal["disable", "enable"] | None = ...,
        webproxy_forward_server: str | None = ...,
        traffic_shaper: str | None = ...,
        traffic_shaper_reverse: str | None = ...,
        per_ip_shaper: str | None = ...,
        nat: Literal["enable", "disable"] | None = ...,
        pcp_outbound: Literal["enable", "disable"] | None = ...,
        pcp_inbound: Literal["enable", "disable"] | None = ...,
        pcp_poolname: str | list[str] | list[dict[str, Any]] | list[PolicyPcppoolnameItem] | None = ...,
        permit_any_host: Literal["enable", "disable"] | None = ...,
        permit_stun_host: Literal["enable", "disable"] | None = ...,
        fixedport: Literal["enable", "disable"] | None = ...,
        port_preserve: Literal["enable", "disable"] | None = ...,
        port_random: Literal["enable", "disable"] | None = ...,
        ippool: Literal["enable", "disable"] | None = ...,
        poolname: str | list[str] | list[dict[str, Any]] | list[PolicyPoolnameItem] | None = ...,
        poolname6: str | list[str] | list[dict[str, Any]] | list[PolicyPoolname6Item] | None = ...,
        session_ttl: str | None = ...,
        vlan_cos_fwd: int | None = ...,
        vlan_cos_rev: int | None = ...,
        inbound: Literal["enable", "disable"] | None = ...,
        outbound: Literal["enable", "disable"] | None = ...,
        natinbound: Literal["enable", "disable"] | None = ...,
        natoutbound: Literal["enable", "disable"] | None = ...,
        fec: Literal["enable", "disable"] | None = ...,
        wccp: Literal["enable", "disable"] | None = ...,
        ntlm: Literal["enable", "disable"] | None = ...,
        ntlm_guest: Literal["enable", "disable"] | None = ...,
        ntlm_enabled_browsers: str | list[str] | list[dict[str, Any]] | list[PolicyNtlmenabledbrowsersItem] | None = ...,
        fsso_agent_for_ntlm: str | None = ...,
        groups: str | list[str] | list[dict[str, Any]] | list[PolicyGroupsItem] | None = ...,
        users: str | list[str] | list[dict[str, Any]] | list[PolicyUsersItem] | None = ...,
        fsso_groups: str | list[str] | list[dict[str, Any]] | list[PolicyFssogroupsItem] | None = ...,
        auth_path: Literal["enable", "disable"] | None = ...,
        disclaimer: Literal["enable", "disable"] | None = ...,
        email_collect: Literal["enable", "disable"] | None = ...,
        vpntunnel: str | None = ...,
        natip: str | None = ...,
        match_vip: Literal["enable", "disable"] | None = ...,
        match_vip_only: Literal["enable", "disable"] | None = ...,
        diffserv_copy: Literal["enable", "disable"] | None = ...,
        diffserv_forward: Literal["enable", "disable"] | None = ...,
        diffserv_reverse: Literal["enable", "disable"] | None = ...,
        diffservcode_forward: str | None = ...,
        diffservcode_rev: str | None = ...,
        tcp_mss_sender: int | None = ...,
        tcp_mss_receiver: int | None = ...,
        comments: str | None = ...,
        auth_cert: str | None = ...,
        auth_redirect_addr: str | None = ...,
        redirect_url: str | None = ...,
        identity_based_route: str | None = ...,
        block_notification: Literal["enable", "disable"] | None = ...,
        custom_log_fields: str | list[str] | list[dict[str, Any]] | list[PolicyCustomlogfieldsItem] | None = ...,
        replacemsg_override_group: str | None = ...,
        srcaddr_negate: Literal["enable", "disable"] | None = ...,
        srcaddr6_negate: Literal["enable", "disable"] | None = ...,
        dstaddr_negate: Literal["enable", "disable"] | None = ...,
        dstaddr6_negate: Literal["enable", "disable"] | None = ...,
        ztna_ems_tag_negate: Literal["enable", "disable"] | None = ...,
        service_negate: Literal["enable", "disable"] | None = ...,
        internet_service_negate: Literal["enable", "disable"] | None = ...,
        internet_service_src_negate: Literal["enable", "disable"] | None = ...,
        internet_service6_negate: Literal["enable", "disable"] | None = ...,
        internet_service6_src_negate: Literal["enable", "disable"] | None = ...,
        timeout_send_rst: Literal["enable", "disable"] | None = ...,
        captive_portal_exempt: Literal["enable", "disable"] | None = ...,
        decrypted_traffic_mirror: str | None = ...,
        dsri: Literal["enable", "disable"] | None = ...,
        radius_mac_auth_bypass: Literal["enable", "disable"] | None = ...,
        radius_ip_auth_bypass: Literal["enable", "disable"] | None = ...,
        delay_tcp_npu_session: Literal["enable", "disable"] | None = ...,
        vlan_filter: str | None = ...,
        sgt_check: Literal["enable", "disable"] | None = ...,
        sgt: str | list[str] | list[dict[str, Any]] | list[PolicySgtItem] | None = ...,
        internet_service_fortiguard: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicefortiguardItem] | None = ...,
        internet_service_src_fortiguard: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservicesrcfortiguardItem] | None = ...,
        internet_service6_fortiguard: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6fortiguardItem] | None = ...,
        internet_service6_src_fortiguard: str | list[str] | list[dict[str, Any]] | list[PolicyInternetservice6srcfortiguardItem] | None = ...,
        vdom: str | bool | None = ...,
        error_mode: Literal["raise", "return", "print"] | None = ...,
        error_format: Literal["detailed", "simple", "code_only"] | None = ...,
    ) -> FortiObject: ...
    
    # Helper methods
    @staticmethod
    def help(field_name: str | None = ...) -> str: ...
    
    @staticmethod
    def fields(detailed: bool = ...) -> list[str] | list[dict[str, Any]]: ...
    
    @staticmethod
    def field_info(field_name: str) -> FortiObject: ...
    
    @staticmethod
    def validate_field(name: str, value: Any) -> bool: ...
    
    @staticmethod
    def required_fields() -> list[str]: ...
    
    @staticmethod
    def defaults() -> FortiObject: ...
    
    @staticmethod
    def schema() -> FortiObject: ...


__all__ = [
    "Policy",
    "PolicyPayload",
    "PolicyResponse",
    "PolicyObject",
]